package nu.fw.jeti.plugins.drawing.elements;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import nu.fw.jeti.backend.XExecutableExtension;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.Packet;
import nu.fw.jeti.plugins.drawing.Plugin;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.util.Log;


/**
 * This class represents the abstract class for picture messages.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public abstract class PictureMessage extends Extension implements XExecutableExtension {
	protected List<Long> shapeIdList;
	
	protected String actionsAuthor;

	protected String thread;

	protected String subject;

	protected String time;

	protected int counter;
	
	private JID to;
	

	/**
	 * Tag used to send the shape's id.
	 */
	public static final String SHAPE_ID_TAG = "shapeId";
	
	/**
	 * Tag used to send the action's author.
	 */
	public static final String ACTIONS_AUTHOR_TAG = "actions_author";

	/**
	 * Tag used to send the chat's thread.
	 */
	public static final String THREAD_TAG = "thread";

	/**
	 * Tag used to send the chat's subject.
	 */
	public static final String SUBJECT_TAG = "subject";

	/**
	 * Tag used to send the shape's color.
	 */
	public static final String COLOR_TAG = "color";

	/**
	 * Tag used to send the message's time.
	 */
	public static final String TIME_TAG = "time";

	/**
	 * Tag used to send the message's counter.
	 */
	public static final String COUNTER_TAG = "counter";

	/**
	 * Tag used to this is a message.
	 */
	public static final String MESSAGE_TAG = "message";

	/**
	 * Tag used to indicate this is a draw message.
	 */
	public static final String DRAW_MESSAGE_TAG = "draw_mess";

	/**
	 * Constructs the message from the builder.
	 * 
	 * @param pmb
	 *            builder where all needed fields are set
	 */
	public PictureMessage(PictureMessageBuilder pmb) {
		shapeIdList = stringToLongList(pmb.shapeIdList);
		actionsAuthor = pmb.actionsAuthor;
		thread = pmb.thread;
		subject = pmb.subject;
		time = pmb.time;
		counter = Integer.parseInt(pmb.counter);
	}

	/**
	 * Constructs the message and sets all the fields.
	 * 
	 * @param to
	 *            adresser of the message
	 * @param author
	 * 			  action's author
	 * @param idList
	 *            shapes' id list
	 * @param t
	 *            thread of the chat
	 * @param time
	 *            time of the message
	 * @param c
	 *            counter of the message
	 */
	public PictureMessage(JID to,String author, List<Long> idList,
			String thread,String time, int c) {
		actionsAuthor = author;
		shapeIdList = idList;
		this.time = time;
		counter = c;
		this.to = to;
		this.thread = thread;
	}

	/**
	 * Converts String with 4 integers to Color with alpha value.
	 * 
	 * @param c
	 *            String with 4 integers
	 * @return returns created Color.
	 */
	public static Color stringToColor(String c) {
		String[] rgb = c.split(" ");
		return new Color(Integer.parseInt(rgb[0]), Integer.parseInt(rgb[1]),
				Integer.parseInt(rgb[2]), Integer.parseInt(rgb[3]));
	}

	/**
	 * Converts the Color with alpha value to String.
	 * 
	 * @param c
	 *            Color to be converted
	 * @return String wih 4 integers
	 */
	public static String colorToString(Color c) {
		return c.getRed() + " " + c.getGreen() + " " + c.getBlue() + " "
				+ c.getAlpha();
	}

	protected List<Long> stringToLongList(String string) {
		if (string != null && !string.equals("")) {
			String[] result = (string.split("\\s"));
			List<Long> idList = new ArrayList<Long>();
			for (int i = 0; i < result.length; i++) {
				idList.add(Long.parseLong(result[i]));
			}
			return idList;
		} else {
			return null;
		}
	}

	protected JID stringToJID(String s) {
		JID jid = null;
		try {
			jid = JID.checkedJIDFromString(s);
		} catch (InstantiationException e) {
			Log.notParsedXML("CreationMessage: Shape's author is not JID, but "
					+ s);
		}
		return jid;
	}
	
	public void execute(Packet packet, Backend backend)
	{
		PictureChat chat = Plugin.getPictureChat(packet.getFrom(), getThread());
		chat.newMessage(this);
	}
	

	protected void startAppendingToXML(StringBuffer xml, String typeTag) {
		startAppendingToXMLWithoutShapeId(xml, typeTag);
		appendElement(xml, SHAPE_ID_TAG, longListToString(shapeIdList));
	}

	protected void startAppendingToXMLWithoutShapeId(StringBuffer xml,
			String typeTag) {
		xml.append("<data xmlns= 'http://jeti.sf.net/protocol/whiteboard'");
		//appendBaseAttributes(xml);
		appendAttribute(xml, DRAW_MESSAGE_TAG, "true");
		appendAttribute(xml, TIME_TAG, time);
		appendAttribute(xml, COUNTER_TAG, counter);
		xml.append(">");
		appendElement(xml, THREAD_TAG, thread);
		appendElement(xml, ACTIONS_AUTHOR_TAG, actionsAuthor);
		// todo there is no subjec; do we need it?
		xml.append("<" + typeTag + ">");
	}

	private String longListToString(List<Long> list) {
		String s = "";
		if (list != null) {
			for (Long id : list) {
				s = s + id + " ";
			}
		}
		return s;
	}

	protected void endAppendingToXML(StringBuffer xml, String typeTag) {
		xml.append("</" + typeTag + ">");
		//appendExtensions(xml);
		xml.append("</data>");
	}

	public List<Long> getShapeIdList() {
		return shapeIdList;
	}
	
	public String getActionsAuthor() {
		return actionsAuthor;
	}

	public String getThread() {
		return thread;
	}
	
	public JID getTo() {
		return to;
	}

	public int getCounter() {
		return counter;
	}

	public String getTime() {
		return time;
	}
}
