package nu.fw.jeti.plugins.drawing.geometry;

import java.awt.geom.Point2D;

/**
 * This class provides simple operations on mathematical vectors.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Vector {
	private double x, y;

	/**
	 * Constructs a new vector with the specified coordinates.
	 * 
	 * @param pX
	 *            coordinate x
	 * @param pY
	 *            coordinate y
	 */
	public Vector(double pX, double pY) {
		x = pX;
		y = pY;
	}

	/**
	 * Constructs a new vector representing vector AB.
	 * 
	 * @param A
	 *            first point of new vector
	 * @param B
	 *            second point of new vector
	 */
	public Vector(Point2D.Double A, Point2D.Double B) {
		x = B.x - A.x;
		y = B.y - A.y;
	}

	/**
	 * Adds the vector to the point.
	 * 
	 * @param A
	 *            a point to be changed
	 * @return the point changed by the vector
	 */
	public Point2D.Double addVectorToPoint(Point2D.Double A) {
		return new Point2D.Double(A.x + x, A.y + y);
	}

	/**
	 * Sets the vector to (0,0).
	 * 
	 */
	public void clear() {
		x = 0;
		y = 0;
	}

	/**
	 * Caculates the cross product of 2 vectors.
	 * 
	 * @param v
	 *            first vector
	 * @param w
	 *            second vector
	 * @return the cross product
	 */
	public static double crossProduct(Vector v, Vector w) {
		return (v.x * w.y - v.y * w.x);
	}

	/**
	 * Caculates the dot product of 2 vectors.
	 * 
	 * @param v
	 *            first vector
	 * @param w
	 *            second vector
	 * @return the dot product
	 */
	public static double dotProduct(Vector v, Vector w) {
		return (v.x * w.x + v.y * w.y);
	}

	/**
	 * Checks if all the points lie on one line.
	 * 
	 * @param A
	 *            first point
	 * @param B
	 *            second point
	 * @param C
	 *            third point
	 * @return <tt>true</tt> if the points are on the same line
	 */
	public static boolean pointsInOneLine(Point2D.Double A, Point2D.Double B, Point2D.Double C) {
		Vector AB = new Vector(A, B);
		Vector AC = new Vector(A, C);
		return (crossProduct(AB, AC) == 0);
	}

	/**
	 * Checks if the angle created between two vectors is less or equal 90
	 * degrees.
	 * 
	 * @param v
	 *            first vector
	 * @param w
	 *            second vector
	 * @return <tt>true</tt> if the angle is less or equal 90 degrees
	 */
	public static boolean isAngleBetweenVectorsLessEqual90(Vector v, Vector w) {
		return (dotProduct(v, w) >= 0);
	}

	/**
	 * Checks if the angle created between two vectors is less or equal 90
	 * degrees. The vectors are created by points BA and AX. We are looking at
	 * the angle BAX.
	 * 
	 * @param B
	 *            first point of the angle
	 * @param A
	 *            the corner of the angle
	 * @param X
	 *            third point of the angle
	 * @return <tt>true</tt> if the angle is less or equal 90 degrees
	 */
	public static boolean isAngleBetweenVectorsLessEqual90(Point2D.Double B, Point2D.Double A,
			Point2D.Double X) {
		Vector AB = new Vector(A, B);
		Vector AX = new Vector(A, X);
		return isAngleBetweenVectorsLessEqual90(AB, AX);
	}

	/**
	 * Calculates distance between two points.
	 * 
	 * @param A
	 *            first point
	 * @param B
	 *            second point
	 * @return the distance between two points
	 */
	public static double distance(Point2D.Double A, Point2D.Double B) {
		return Math.sqrt((A.x - B.x) * (A.x - B.x) + (A.y - B.y) * (A.y - B.y));
	}

	/**
	 * Calculates the length of the vector.
	 * 
	 * @param v
	 *            vector
	 * @return the length of the vector
	 */
	public static double length(Vector v) {
		return Math.sqrt(v.x * v.x + v.y * v.y);
	}

	/**
	 * Calculates the distance between the line and the point. The line is
	 * created by two points: A and B.
	 * 
	 * @param A
	 *            first point on the line
	 * @param B
	 *            second point on the line
	 * @param X
	 *            the point for which we calculate the distance
	 * @return the distance between the line and the point
	 */
	public static double distanceLinePoint(Point2D.Double A, Point2D.Double B, Point2D.Double X) {
		Vector AX = new Vector(A, X);
		Vector AB = new Vector(A, B);
		double distanceAB = length(AB);
		if (distanceAB == 0) {
			return 1000000;
			// throw new IllegalArgumentException("A=B");
		} else {
			return Math.abs(crossProduct(AX, AB) / distanceAB);
		}
	}

	/**
	 * Gets the vector with the reversed coordinates.
	 * 
	 * @return the vector -v
	 */
	public Vector getReverseVector() {
		return new Vector(-x, -y);
	}

	/**
	 * Returns a string representation of this Vector, containing the String
	 * representation of the coordinates.
	 */
	public String toString() {
		return x + " " + y;
	}
}
