package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

/**
 * This class represents the ellipse.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Ellipse extends Shape {
	private double x;

	private double y;

	private double width;

	private double height;

	/**
	 * Tag used to send this shape.
	 */
	public static final String NAME = "ellipse";

	/**
	 * Constructs a new ellipse and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param author
	 *            shape's author
	 * @param coords
	 *            shape's coordinates converted to String
	 * @param color
	 *            shape's color
	 * @param displayHeight
	 *            shape's height in the picture's array
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 */
	public Ellipse(long id, JID author, String coords, Color color, int displayHeight,
			boolean filled) {
		super(id, author, color, displayHeight);
		setCoordinates(coords);
		isFilled = filled;
	}

	/**
	 * Constructs a new ellipse and sets all the fields.
	 * 
	 * @param x2
	 *            upper left corner of the rectangle describing the ellipse
	 * @param y2
	 *            bottom right corner of the rectangle describing the ellipse
	 * @param w
	 *            rectangle's width
	 * @param h
	 *            rectangle's height
	 * @param color
	 *            shape's color
	 * @param author
	 *            shape's author
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 */
	public Ellipse(double x2, double y2, double w, double h, Color color, JID author,
			boolean filled) {
		super(generateId(), author, color, INCORRECT_HEIGHT);
		isFilled = filled;
		x = x2;
		y = y2;
		width = w;
		height = h;
	}

	public void draw(Graphics g, int xScale, int yScale) {
		int newX = ceil(x, xScale);
		int newY = ceil(y, yScale);
		int newWidth = ceil(width, xScale);
		int newHeight = ceil(height, yScale);
		
		if (isFilled) {
			g.setColor(color);
			g.fillOval(newX, newY, newWidth+1, newHeight+1);
		}
		if (isFilled) {
			g.setColor(borderColor);
		} else {
			g.setColor(color);
		}
		g.drawOval(newX, newY, newWidth, newHeight);
	}

	public void drawAsSelected(Graphics g, int xScale, int yScale) {
		draw(g, xScale, yScale);
		int newX = ceil(x, xScale);
		int newY = ceil(y, yScale);
		int newWidth = ceil(width, xScale);
		int newHeight = ceil(height, yScale);
		
		g.setColor(selectionColor);
		g.drawRect(newX - 2, newY - 2, 4, 4);
		g.drawRect(newX - 2, newY + newHeight - 2, 4, 4);
		g.drawRect(newX + newWidth - 2, newY - 2, 4, 4);
		g.drawRect(newX + newWidth - 2, newY + newHeight - 2, 4, 4);
	}

	public String getName() {
		return NAME;
	}

	public String getCoordinates() {
		return x + " " + y + " " + width + " " + height;
	}

	/**
	 * @param coords
	 *            coordinates in String: upper left corner of the rectangle,
	 *            bottom right corner of the rectangle, width, height
	 */
	public void setCoordinates(String coords) {
		String[] result = coords.split("\\s");
		int i = 0;
		x = Double.parseDouble(result[i++]);
		y = Double.parseDouble(result[i++]);
		width = Double.parseDouble(result[i++]);
		height = Double.parseDouble(result[i++]);
	}

	public void changeCoordinates(Vector v) {
		Point2D.Double p = v.addVectorToPoint(new Point2D.Double(x, y));
		x = p.x;
		y = p.y;
	}

	public boolean isInsideTheRect(Point2D.Double upperLeft, Point2D.Double bottomRight) {
		return (PointsOperations.pointIsInsideRect(new Point2D.Double(x, y), upperLeft,
				bottomRight) && PointsOperations.pointIsInsideRect(new Point2D.Double(x
				+ width, y + height), upperLeft, bottomRight));
	}

	public boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		if (isFilled) {
			return isPointInShapesInterior(p, scaleX, scaleY);
		} else {
			int a = ceil((double)(width / 2), scaleX);
			int b = ceil((double)(height / 2), scaleY);
			int newX = ceil(x, scaleX);
			int newY = ceil(y, scaleY);
			
			Point scaledPoint = ceil(p, scaleX, scaleY);			
			int px = scaledPoint.x - (newX + a);
			int py = scaledPoint.y - (newY + b);
			
			if (a > SURROUND_DIST || b > SURROUND_DIST) {
				double r1 = px / (a - SURROUND_DIST);
				double r2 = py / (b - SURROUND_DIST);
				if (r1 * r1 + r2 * r2 < 1) {
					return false;
				}
			}
			double r1 = px / (a + SURROUND_DIST);
			double r2 = py / (b + SURROUND_DIST);
			return (r1 * r1 + r2 * r2 <= 1);
		}
	}

	public Point2D.Double getDisplayPoint() {
		return new Point2D.Double(x + width / 2, y + height / 2);
	}

	public Point2D.Double getTheMostUpperLeftPoint() {
		return new Point2D.Double(x, y);
	}

	public Point2D.Double getTheMostBottomRightPoint() {
		return new Point2D.Double(x + width, y + height);
	}

	@Override
	public boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY) {
		int newX  = ceil(x, scaleX);
		int newY = ceil(y, scaleY);
		int newWidth = ceil(width, scaleX);
		int newHeight = ceil(height, scaleY);
		Point2D.Double scaledPoint = ceilTo2DPointl(p, scaleX, scaleY);
		Point2D.Double upperLeft = new Point2D.Double(newX - SURROUND_DIST, newY - SURROUND_DIST);
		Point2D.Double bottomRight = new Point2D.Double(newX + newWidth - SURROUND_DIST,
				newY + newHeight - SURROUND_DIST);
		return PointsOperations.pointIsInsideRect(scaledPoint, upperLeft, bottomRight);
	}
}
