package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.*;

import javax.imageio.ImageIO;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.SwingUtilities;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.elements.CurrentUsersMessage;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.geometry.Vector;
import nu.fw.jeti.plugins.drawing.shapes.actions.Action;
import nu.fw.jeti.plugins.drawing.ui.HistoryPanel;
import nu.fw.jeti.plugins.drawing.ui.PictureChatFrame;
import nu.fw.jeti.plugins.drawing.ui.PictureFileFilter;
import nu.fw.jeti.plugins.drawing.ui.UserRepresentation;

/**
 * This class is a representation of one drawing chat. This is the main class
 * for the chat where we can draw.  The class object is created when a user
 * starts a drawing chat or if another user starts it and sends something to
 * him.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PictureChat {
	private PictureChatFrame window;

	private Backend backend;

	private List<JID> receivers; // with who we draw

	private String thread;

	private List<PicturesContent> pictures;

	private PicturesContent currentPicture;

	private PictureHistory history;

	private Map<String, ImageWriter> writersMap;

	private boolean isBlockedChat = false;

	private List<Shape> staticClipboard = Collections
			.synchronizedList(new ArrayList<Shape>());

	/**
	 * Constructs a picture chat.
	 * 
	 * @param b
	 *            backend; the class takes informations about the whole
	 *            application from backend
	 * @param userName
	 *            the first user whith who we talk
	 * @param t
	 *            this chat's thread
	 */
	public PictureChat(Backend b, JID userName, String t) {
		pictures = Collections
				.synchronizedList(new ArrayList<PicturesContent>());
		currentPicture = new PicturesContent();
		pictures.add(currentPicture);
		receivers = Collections.synchronizedList(new ArrayList<JID>());

		backend = b;
		HistoryPanel historyPanel = new HistoryPanel(this);
		history = new PictureHistory(backend, historyPanel);

		receivers.add(userName);
		thread = t;
		window = new PictureChatFrame(this, currentPicture, historyPanel);
		window.setCurrentUsers(receivers);

		writersMap = createWritersMap();

		showWindow();
	}

	private Map<String, ImageWriter> createWritersMap() {
		Map<String, ImageWriter> map = Collections
				.synchronizedMap(new HashMap<String, ImageWriter>());
		List<String> extList = PictureFileFilter.getAcceptedExt();
		for (String ext : extList) {
			ImageWriter w = getWriter(ext);
			if(w!=null)map.put(ext,w);
		}
		return map;
	}

	private ImageWriter getWriter(String ext) {
		Iterator writers = ImageIO.getImageWritersByFormatName(ext);
		if(writers.hasNext()) return (ImageWriter) writers.next();
		return null;
	}

	/**
	 * Processes the incoming message.
	 * 
	 * @param m
	 *            incoming message
	 */
	public void newMessage(final PictureMessage m) {
		if (isBlockedChat) {
			return;
		}
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (m instanceof CurrentUsersMessage) {
					updateUsers(((CurrentUsersMessage) m).getUsers());
				} else {
					history.addIncomingMessage(m, pictures);
					showWindow();
				}
			}
		});
	}

	/**
	 * Processes the shape drawn by me.  It draws it, puts to the history and
	 * sends to other users.
	 * 
	 * @param shape
	 *            drawn shape
	 */
	public void draw(Shape shape) {
		history.addShapeDrawnByMe(shape, getMyJID().toString(), pictures,
				receivers, thread);
	}

	/**
	 * Processes the deletion done by me.  It deletes the shape, puts the deletion
	 * to the history and sends it to other users.
	 * 
	 * @param shapeId
	 *            deleted shape's id
	 */
	public void delete(long shapeId) {
		history.addDeletionDoneByMe(Action.createShapeIdList(shapeId),
				getMyJID().toString(), pictures, receivers, thread);
	}

	/**
	 * Processes the undo done by me.  It udnos the last action and sends it to
	 * the other users.
	 */
	public void undo() {
		history.undoMyAction(pictures, receivers, thread);
	}

	/**
	 * Processes the redo done by me.  It rodos the last action and sends it to
	 * the other users.
	 */
	public void redo() {
		history.redoMyAction(pictures, receivers, thread);
	}

	// TODO trzeba to zmienic; mozna dac, ze kilka czatow z jedna osoba sie ma
	// itp.
	/**
	 * @deprecated Gets the first user's JID.
	 */
	public JID getJID() {
		return receivers.get(0);
	}

	/**
	 * Gets my JID
	 * @return my JID
	 */
	public JID getMyJID() {
		return backend.getMyJID();
	}

	/**
	 * Shows the piture chat window.
	 *
	 */
	public void showWindow() {
		if (!window.isVisible()) {
			window.setVisible(true);
		}
		window.repaint();
	}

	/**
	 * Selects the shapes which are inside the rectangle determined by
	 * Points p1 and p2.
	 * 
	 * @param p1 first Point of the selected area
	 * @param p2 second Point of the selected area
	 */
	public void selectItems(Point2D.Double p1, Point2D.Double p2) {
		currentPicture.selectItems(p1, p2);
	}

	/**
	 * Selects the newest shape which is in the Point's neighbourhood.
	 * @param p Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 */
	public void selectItem(Point2D.Double p, int scaleX, int scaleY) {
		currentPicture.selectItem(p, scaleX, scaleY);
	}

	/**
	 * Unselects all the shapes.
	 */
	public void unselect() {
		currentPicture.unselect();
	}

	/**
	 * Checks if the Point is in the neighbourhood of any of the selected
	 * shapes.
	 * 
	 * @param p Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return <tt>true</tt> if the Point is in the neighbourhood of any of the
	 * selected shapes
	 */
	public boolean checkIfClickedInSelection(Point2D.Double p, int scaleX, int scaleY) {
		return currentPicture.checkIfClickedInSelection(p, scaleX, scaleY);
	}

	/**
	 * Deletes selected shapes, puts the deletion to the history and sends it
	 * to the other users.
	 */
	public void deleteSelectedShapes() {
		List<Long> list = currentPicture.getSelectedShapesIdList();
		currentPicture.unselect();
		history.addDeletionDoneByMe(list, getMyJID().toString(), pictures,
				receivers, thread);
	}

	/**
	 * Gets the author of the shape, which is in the Point's neighbourhood.
	 * 
	 * @param point Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return the shape's author's JID
	 */
	public JID getAuthor(Point2D.Double point, int scaleX, int scaleY) {
		Shape shape = currentPicture.getShapeFromNeighbourhood(point, scaleX, scaleY);
		if (shape != null) {
			return shape.getAuthor();
		} else {
			return null;
		}

	}

	/**
	 * Moves all the selected shapes by the Vector.
	 * @param vector Vector by which the shapes will be translated
	 */
	public void moveShapes(Vector vector) {
		currentPicture.moveShapes(vector);

	}

	/**
	 * Puts the translation of the selected shapes to the history and sends
	 * it to the other users.  Does not translate the shapes on my picture (it
	 * was already done by moving the mouse).
	 * @param vector Vector by which the shapes are translated
	 */
	public void sendShapesTranslation(Vector vector) {
		history.addTranslationDoneByMe(currentPicture.getSelectedShapesIdList(),
				vector, getMyJID().toString(),
				pictures, receivers, thread);
	}

	/**
	 * Adds new users to the conversation. Sends the new users' list to all the
	 * users and sends the history to new users.
	 * @param selectedUsers new users that are added
	 */
	public void addUsers(Set<UserRepresentation> selectedUsers) {
		List<JID> newUsers = new ArrayList<JID>();
		for (UserRepresentation user : selectedUsers) {
			JID jid = user.getJID();
			if (thereIsNoSuchUserYet(jid)) {
				newUsers.add(jid);
			}
		}
		if (!newUsers.isEmpty()) {
			history.sendAllHistory(newUsers, thread);
			receivers.addAll(newUsers);
			List<JID> currentUsers = new ArrayList<JID>();
			currentUsers.addAll(receivers);
			currentUsers.add(getMyJID());
			history.sendCurrentUsers(getMyJID().toString(), currentUsers,
					receivers, thread);
		}

		window.setCurrentUsers(receivers);

	}

	/**
	 * Checks if the user is already active in chat.
	 * @param jid checked user's JID 
	 * @return <tt>true</tt> if there is not such a user yet
	 */
	public boolean thereIsNoSuchUserYet(JID jid) {
		if (jid.getUser() == null) {
			return false;
		}
		return (!receivers.contains(jid) &&
				!jid.getUser().equals(getMyJID().getUser()));
	}

	/**
	 * Gets backend.
	 * @return backend
	 */
	public Backend getBackend() {
		return backend;
	}

	private void updateUsers(List<JID> users) {
		receivers = users;
		receivers.remove(getMyJID());
		window.setCurrentUsers(receivers);
	}

	/**
	 * Sets the selected shapes color. Puts it to the history and sends it to
	 * the other users.
	 * @param color new color to be set
	 */
	public void setShapeColor(Color color) {
		history.addColorSetting(currentPicture.getSelectedShapesIdList(),
				getMyJID().toString(), color, pictures, receivers, thread);
	}

	/**
	 * Sets background color. Puts it to the history and sends it to
	 * the other users.
	 * @param newColor actual color of the background
	 */
	public void setBackgroundColor(Color newColor) {
		Color oldColor = currentPicture.getBackgroundColor();
		history.addBackgroundColorSetting(getMyJID().toString(), oldColor,
				newColor, pictures,	receivers, thread);
	}

	/**
	 * Deletes all the shapes from the picture. Puts it to the history
	 * and sends it to the other users.
	 */
	public void clearBoard() {
		currentPicture.selectAll();
		deleteSelectedShapes();
		setBackgroundColor(PicturesContent.DEFAULT_BACKGROUND_COLOR);
	}

	/**
	 * Selects all shapes from the picture.
	 */
	public void selectAll() {
		currentPicture.selectAll();
	}

	/**
	 * Gets the text shape which is in the neighbourhood of the Point. 
	 * @param point Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return the clicked text shape<br>
	 * <tt>null</tt> if none text shape was clicked
	 */
	public TextShape getClickedTextShape(Point2D.Double point, int scaleX, int scaleY) {
		return currentPicture.getClickedTextShape(point, scaleX, scaleY);
	}

	/**
	 * Saves the current picture to the given file.
	 * @param file file to which it will save the picture
	 * @param width TODO
	 * @param height TODO
	 */
	public void savePicture(File file, int width, int height) {
		// we should receive only good files here
		ImageWriter writer = writersMap.get(PictureFileFilter
				.getExtension(file));
		if (writer == null) {
			throw new IllegalArgumentException("PictureChat: File did not have"
					+ " a good extension: " + file + ". For proper extentions"
					+ " look to PictureFileFilter");
		}
		try {
			setWriterOutput(file, writer);
		} catch (IOException e1) {
			System.out.println("PictureChat: Unable to create an output stream.");
			return;
		}

//		Point2D.Double p = currentPicture.getTheMostBottomRightPoint();
//		int width = (int)p.x;
//		int height = (int)p.y;
		BufferedImage bi = new BufferedImage(width, height,
				BufferedImage.TYPE_3BYTE_BGR);

		Graphics g = bi.getGraphics();
		drawPicture(width, height, g);

		try {
			writer.write(bi);
		} catch (IOException e) {
			System.out.println("PictureChat: Unable to write BufferedImage"
					+ " on the ImageWriter.");
			return;
		}
	}

	private void setWriterOutput(File file, ImageWriter writer)
			throws IOException {
		ImageOutputStream ios = null;
		ios = ImageIO.createImageOutputStream(file);

		writer.setOutput(ios);
	}

	private void drawPicture(int width, int height, Graphics g) {
		g.setColor(window.getCurrentBackgroundColor());
		g.fillRect(0, 0, width, height);
		currentPicture.drawPicture(g, width, height);
	}

	/**
	 * Copies the selected shapes.
	 */
	public void copy() {
		staticClipboard.clear();
		staticClipboard.addAll(currentPicture.getCurrentlySelected());
	}

	/**
	 * Pastes the previously copied shapes. Puts it to the history and sends
	 * to other users.
	 */
	public void paste() {
		if (!staticClipboard.isEmpty()) {
			Point2D.Double p = currentPicture
					.getTheMostUpperLeftPointAmongGaveList(staticClipboard);
			Vector v = new Vector(-p.x, -p.y);
			history.addPasteDoneByMe(staticClipboard, v, 
					getMyJID().toString(), pictures, receivers,	thread);
		}
	}

	/**
	 * Blocks this chat.  There will be no more messages incoming and none will
	 * be sent.
	 */
	public void blockChat() {
		isBlockedChat = true;
		receivers.clear();
	}

	/**
	 * Cuts all the selected shapes (copies and deletes them).
	 */
	public void cut() {
		staticClipboard.clear();
		staticClipboard.addAll(currentPicture.getCurrentlySelected());
		deleteSelectedShapes();
	}

}
