package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.text.DateFormat;
import java.util.*;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Message;
import nu.fw.jeti.plugins.drawing.Plugin;
import nu.fw.jeti.plugins.drawing.elements.*;
import nu.fw.jeti.plugins.drawing.geometry.Vector;
import nu.fw.jeti.plugins.drawing.shapes.actions.*;
import nu.fw.jeti.plugins.drawing.ui.HistoryPanel;

/**
 * This class represents the history of the changes done on the picture.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PictureHistory {
	private Map<Long, Shape> allShapes;
	private UndoRedoStackRepresentation myActions;
	private ActionPriorityStack historyStack;
	private Backend backend;
	private HistoryPanel historyPanel;
	private int counter;
	private static final DateFormat DATE_FORMAT = 
		DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.FULL);
	
	/**
	 * Creates a picture history.
	 * @param b backend which gives some information to the history
	 * @param hPanel TODO
	 */
	public PictureHistory(Backend b, HistoryPanel hPanel) {
		backend = b;
		historyPanel = hPanel;
		allShapes = Collections.synchronizedMap(new HashMap<Long, Shape>());
		myActions = new UndoRedoStackRepresentation();
		historyStack = new ActionPriorityStack();
		counter = 0;
	}
	
	/**
	 * Creates an action apropriate to the incoming message.  Then adds
	 * the action to the history stack and executes it on the given
	 * pictures.
	 * @param m incoming message
	 * @param shapeBoards list of pictures on which it should execute
	 * the action
	 */
	public void addIncomingMessage(PictureMessage m, List<PicturesContent> shapeBoards) {
		Action a = null;
		if (m instanceof CreationMessage) {
			Shape shape = createShapeFromCreationMessage((CreationMessage)m);
			a = new Creation(shape, allShapes, m.getTime(), 
					m.getActionsAuthor(), m.getCounter());
		} else if (m instanceof DisplayMessage) {
			a = new Display(m.getShapeIdList(), allShapes, m.getTime(),
					m.getActionsAuthor(), m.getCounter());
		} else if (m instanceof DeletionMessage) {
			a = new Deletion(m.getShapeIdList(), allShapes, m.getTime(),
					m.getActionsAuthor(), m.getCounter());
		} else if (m instanceof TranslationMessage) {
			a = new Translation(m.getShapeIdList(), allShapes,
					((TranslationMessage)m).getVector(), m.getTime(),
					m.getActionsAuthor(), m.getCounter());
		} else if (m instanceof ColorSettingMessage) {
			a = new ColorSetting(m.getShapeIdList(), allShapes,
					((ColorSettingMessage)m).getColor(), m.getTime(),
					m.getActionsAuthor(), m.getCounter());
		} else if (m instanceof BackgroundSettingMessage) {
			a = new BackgroundColorSetting(((BackgroundSettingMessage)m).getOldColor(),
					((BackgroundSettingMessage)m).getColor(), m.getTime(),
					m.getActionsAuthor(), m.getCounter());					
		}
		
		if (a != null) {
			historyStack.add(a, shapeBoards);
			addActionToHistoryPanel(a);
			for (PicturesContent board : shapeBoards) {
				a.execute(board);
			}
		} else {
			throw new IllegalArgumentException("PictureHistory: The message " + m +
					" service is not implemented yet.");
		}
		
		increaseCounter(m);
	}

	private void addActionToHistoryPanel(Action a) {
		historyPanel.addAction(a);
	}

	private void increaseCounter(PictureMessage m) {
		counter = Math.max(counter, m.getCounter() + 1);
	}
	
	private void increaseCounter(int addedValue) {
		counter += addedValue;
	}

	/**
	 * Creates a new {@link Creation} and {@link Display} action, sends
	 * them to other users and exectutes them.
	 * 
	 * @param s drawn shape
	 * @param actionsAuthor the author of the action
	 * @param shapeBoards list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void addShapeDrawnByMe(Shape s, String actionsAuthor,
			List<PicturesContent> shapeBoards, 
			List<JID> receivers, String thread) {
		if (s == null) {
			return;
		}
		s.setDisplayHeight(counter);
		String time = generateTime();

		Action creation = new Creation(s, allShapes, time,
				actionsAuthor, counter);
		historyStack.add(creation, shapeBoards);
		sendMyAction(receivers, thread, creation);
		executeMyAction(shapeBoards, creation);
		addActionToHistoryPanel(creation);
		// we don't put it on the undo/redo stack
		
		List<Long> shapeIdList = Action.createShapeIdList(s.getShapeId());
		Action display = new Display(shapeIdList, allShapes, time,
				actionsAuthor, counter);
		serviceMyAction(shapeBoards, receivers, thread, display);
	}
	
	private String generateTime() {
		return DATE_FORMAT.format(new Date());
	}

	/**
	 * Creates a {@link Deletion} action, send it to the other users
	 * and executes it.
	 * @param sIdList which shapes will be deleted
	 * @param actionsAuthor Action's author
	 * @param shapeBoards list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void addDeletionDoneByMe(List<Long> sIdList, String actionsAuthor,
			List<PicturesContent> shapeBoards, 
			List<JID> receivers, String thread) {
		if (sIdList.isEmpty()) {
			return;
		}
		Action a = new Deletion(sIdList, allShapes, generateTime(),
				actionsAuthor, counter);
		serviceMyAction(shapeBoards, receivers, thread, a);
	}
	
	/**
	 * Creates a {@link Translation} action, adds it to the history, executes
	 *  and sends it to other users.
	 *  
	 * @param selectedShapesIdList shapes which will be moved
	 * @param vector how much we want to move the shapes
	 * @param actionsAuthor Action's author
	 * @param pictures list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void addTranslationDoneByMe(
			List<Long> selectedShapesIdList, 
			Vector vector,
			String actionsAuthor,
			List<PicturesContent> pictures,
			List<JID> receivers,
			String thread) {
		if (selectedShapesIdList.isEmpty()) {
			return;
		}
		Action a = new Translation(selectedShapesIdList, allShapes,
				vector, generateTime(), actionsAuthor, counter);

		serviceMyAction(pictures, receivers, thread, a);
	}
	
	/**
	 * Creates a {@link CurrentUsersDelivery} action and sends it to other
	 * users.
	 * 
	 * @param actionsAuthor Action's author
	 * @param currentUsers list of currents users on the chat
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void sendCurrentUsers(String actionsAuthor, List<JID> currentUsers,
			List<JID> receivers, String thread) {
		Action a = new CurrentUsersDelivery(currentUsers, generateTime(),
				actionsAuthor, counter);
		sendMyAction(receivers, thread, a);
	}
	
	/**
	 * Adds the action to the stack, informs myActions
	 * that we drew something, sends the message to the receivers, informs
	 * shapeBoards that something is changed on the picture.
	 * 
	 * @param shapeBoards PicturesContent, which are interested in receiving 
	 * information about changing the picture
	 * @param receivers other pictureChat users
	 * @param thread actual thread
	 * @param a done action
	 */
	private void serviceMyAction(List<PicturesContent> shapeBoards, 
			List<JID> receivers, String thread, Action a) {
		historyStack.add(a, shapeBoards);
		sendMyAction(receivers, thread, a);
		executeMyAction(shapeBoards, a);
		myActions.addAction(a);	
		addActionToHistoryPanel(a);
	}
	
	private void executeMyAction(List<PicturesContent> shapeBoards, Action a) {
		for (PicturesContent board : shapeBoards) {
			a.execute(board);
		}
	}

	private void sendMyAction(List<JID> receivers, String thread, Action a) {		
		
		List<PictureMessage> messList = a.getPictureMessageList(receivers, thread);
		for (PictureMessage message : messList) {
			backend.send(new Message(null,message.getTo(), message));
		}
		increaseCounter(1);
		if (a instanceof ColorSetting) {
			increaseCounter(a.getShapesNumber());
		}
	}
	
	/**
	 * Undos my last action.  Takes the last action done by me, creates
	 * a complementary action, adds it to the history, sends it and executes.
	 * Unselects any selected shapes.
	 * 
	 * @param shapeBoards  list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void undoMyAction(List<PicturesContent> shapeBoards, 
			List<JID> receivers, String thread) {
		Action a = myActions.undo();
		if (a != null) {
			Action complementaryAction = a.giveComplementaryAction(generateTime(), counter);
			historyStack.add(complementaryAction, shapeBoards);
			sendMyAction(receivers, thread, complementaryAction);
			executeMyAction(shapeBoards, complementaryAction);	
			addActionToHistoryPanel(complementaryAction);
		}
		unselect(shapeBoards);
	}

	/**
	 * Redos my last undone action.  Adds the redone action to the history,
	 * sends it and executes. Unselects any selected shapes.
	 * 
	 * @param shapeBoards  list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void redoMyAction(List<PicturesContent> shapeBoards, 
			List<JID> receivers, String thread) {
		Action a = myActions.redo();
		if (a != null) {
			a.setCounter(counter);
			historyStack.add(a, shapeBoards);
			sendMyAction(receivers, thread, a);
			executeMyAction(shapeBoards, a);
			addActionToHistoryPanel(a);
		}
		unselect(shapeBoards);
	}
	
	private void unselect(List<PicturesContent> shapeBoards) {
		for (PicturesContent board : shapeBoards) {
			board.unselect();
		}		
	}
	
	private Shape createShapeFromCreationMessage(CreationMessage m) {
		// there should be exactly one shape id in the message
		long shapeId = m.getShapeIdList().get(0);
		Shape s = Plugin.createShape(m.getShapeName(), shapeId,
				m.getAuthor(), m.getCoordinates(), m.getColor(),
				m.getShapeHeight(), m.isFilled());
		return s;
	}

	/**
	 * Sends all the actions that were done by now to all the receivers.
	 * 
	 * @param newUsers receivers of the messages
	 * @param thread chat's thread
	 */
	public void sendAllHistory(List<JID> newUsers, String thread) {
		for (int i = 0; i < historyStack.size(); i++) {
			Action a = historyStack.elementAt(i);
			sendMyAction(newUsers, thread, a);
		}
	}

	/**
	 * Creates a {@link BackgroundColorSetting} action, adds it to the history,
	 * sends it and executes.
	 * 
	 * @param actionsAuthor Action's author
	 * @param oldColor old backgroung color
	 * @param newColor new background color to be set
	 * @param pictures list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void addBackgroundColorSetting(String actionsAuthor, Color oldColor,
			Color newColor,	List<PicturesContent> pictures,
			List<JID> receivers, String thread) {
		Action a = new BackgroundColorSetting(oldColor, newColor, generateTime(),
				actionsAuthor, counter);
		serviceMyAction(pictures, receivers, thread, a);
	}

	/**
	 * Creates a {@link ColorSetting} action, adds it to the history,
	 * sends it and executes.
	 * 
	 * @param selectedShapesIdList shapes on which we set new color
	 * @param actionsAuthor Action's author
	 * @param color color to be set
	 * @param pictures list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void addColorSetting(List<Long> selectedShapesIdList,
			String actionsAuthor, Color color,
			List<PicturesContent> pictures, List<JID> receivers, String thread) {
		if (selectedShapesIdList.isEmpty()) {
			return;			
		}
		Action a = new ColorSetting(selectedShapesIdList, allShapes, color, 
				generateTime(), actionsAuthor, counter);
		serviceMyAction(pictures, receivers, thread, a);
	}

	/**
	 * Creates a {@link Pasting} action, adds it to the history,
	 * sends it and executes.  It creates new Shapes, which are the same as
	 * the old ones and displays them.
	 *  
	 * @param shapeList list of shapes that are pasted
	 * @param vector where the shapes will be moved
	 * @param actionsAuthor Action's author
	 * @param pictures list of pictures on which it should execute
	 * @param receivers receivers of the message
	 * @param thread chat's thread
	 */
	public void addPasteDoneByMe(List<Shape> shapeList, Vector vector, String actionsAuthor,
			List<PicturesContent> pictures, List<JID> receivers, String thread) {
		Action a = new Pasting(shapeList, allShapes, vector,
				generateTime(), actionsAuthor, counter);
		/* increase counter by the number of new Creation actions, counter
		 * for display action will be increased after sending
		 */
		increaseCounter(shapeList.size());
		serviceMyAction(pictures, receivers, thread, a);
	}

}
