/*
 * Created on 2006-02-28
 *
 */
package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.Plugin;
import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;
import nu.fw.jeti.plugins.drawing.ui.UserRepresentation;

/**
 * This class represents the content of the picture: all the shapes with
 * colors.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PicturesContent {
	private ShapePriorityStack contents;
	private List<Shape> currentlySelected;
	private Color backgroundColor;
	
	private List<ActionListener> listeners;
	
	public static final int AUTHOR_DISPLAY = 1;
	private static final double MARGIN_LENGTH = 0.03;
	
	public static final Color DEFAULT_BACKGROUND_COLOR = Color.WHITE;
	
	/**
	 * Creates a new picture.
	 */
	public PicturesContent() {
		contents = new ShapePriorityStack();
		listeners = Collections.synchronizedList(new ArrayList<ActionListener>());
		currentlySelected = Collections.synchronizedList(new ArrayList<Shape>());
		backgroundColor = DEFAULT_BACKGROUND_COLOR;
	}
	
	/**
	 * Adds shape to the picture.  Informs the listeners that the picture
	 * changed.
	 * 
	 * @param s new shape
	 */
	public synchronized void addShape(Shape s) {
		contents.add(s);
		pictureChanged();
	}
	
	/**
	 * Adds list of shapes to the picture.  Informs the listeners that
	 * the picture changed.
	 * 
	 * @param shapeList list of the shapes to be added
	 * @param drawAsSelected <tt>true</tt> if new shapes should be drawn
	 * as selected
	 */
	public synchronized void addShapeList(List<Shape> shapeList, boolean drawAsSelected) {
		for (Shape shape : shapeList) {
			contents.add(shape);
		}
		if (drawAsSelected) {
			unselect();
			currentlySelected = shapeList;
		}
		Shape s = shapeList.get(0);
		String param = s.getAuthor() + " "+
				s.getDisplayPoint().x + " " + s.getDisplayPoint().y ;
		pictureChanged(new ActionEvent(this, AUTHOR_DISPLAY, param));
	}

	private synchronized void pictureChanged(ActionEvent event) {
		for (ActionListener l : listeners) {
			l.actionPerformed(event);
		}		
	}

	public synchronized void drawPictureWithSelection(Graphics g, int xScale, int yScale) {
		drawPicture(g, xScale, yScale);
		for (Shape shape : currentlySelected) {
			shape.drawAsSelected(g, xScale, yScale);
		}
	}

	/**
	 * Draws all the shapes that are actually on the picture.
	 * 
	 * @param g where it should draw
	 */
	public void drawPicture(Graphics g, int xScale, int yScale) {
		setBackground(g, xScale, yScale);
		for (int i = 0; i < contents.size(); i++) {
			Shape shape = contents.get(i);
			shape.draw(g, xScale, yScale);
		}
	}

	public void drawPicture(Graphics g, int xScale, int yScale,
			Set<UserRepresentation> displayedAuthors) {
		setBackground(g, xScale, yScale);
		for (int i = 0; i < contents.size(); i++) {
			Shape shape = contents.get(i);
			JID author = shape.getAuthor();
			for (UserRepresentation user : displayedAuthors) {
				if (author.equals(user.getJID())) {
					shape.draw(g, xScale, yScale);
				}
			}
		}
		
	}
	
	private void setBackground(Graphics g, int width, int height) {
		g.setColor(backgroundColor);
		g.fillRect(0, 0, width, height);
	}
	
	/**
	 * Removes the shape from the picture.  Informs the listeners that
	 * the picture changed.
	 * 
	 * @param shape shape to be deleted
	 */
	public synchronized void deleteShape(Shape shape) {
		contents.remove(shape);
		pictureChanged();
	}
	
	/**
	 * Removes the shapes' list from the picture.  Informs the listeners that
	 * the picture changed.
	 * 
	 * @param shapeList shapes' list to be deleted
	 */
	public synchronized void deleteShapeList(List<Shape> shapeList) {
		for (Shape shape : shapeList) {
			contents.remove(shape);
		}
		pictureChanged();
	}
	
	/**
	 * Informs all the listeners that the picture has changed.  It does not
	 * pass any action.
	 */
	public synchronized void pictureChanged() {
		pictureChanged(null);
	}
	
	/**
	 * Registers the listener to the list which is told about the picture's
	 * change.
	 * 
	 * @param l listener to be added
	 */
	public synchronized void addListener(ActionListener l) {
		listeners.add(l);
	}

	/**
	 * Selects the shapes which are inside the rectangle determined by
	 * Points p1 and p2.  Checks which shapes are inside and puts them to
	 * the currentlySelected list. Informs the listeners that
	 * the picture changed.
	 * 
	 * @param p1 first Point of the selected area
	 * @param p2 second Point of the selected area
	 */
	public synchronized void selectItems(Point2D.Double p1, Point2D.Double p2) {
		unselect();
		Point2D.Double upperLeft = PointsOperations.upperLeftPoint(p1, p2);
		Point2D.Double bottomRight = PointsOperations.bottomRightPoint(p1, p2);
		for (int i = 0; i < contents.size(); i++) {
			Shape shape = contents.get(i);
			if (shape.isInsideTheRect(upperLeft, bottomRight)) {
				currentlySelected.add(shape);
			}
		}
		pictureChanged();
	}
	
	/**
	 * Gets the list of currently selected shapes.
	 * 
	 * @return list of currently selected shapes
	 */
	public List<Shape> getCurrentlySelected() {
		return currentlySelected;
	}
	
	/**
	 * Selects the newest shape which is in the Point's neighbourhood.
	 * 
	 * @param p Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 */
	public synchronized void selectItem(Point2D.Double p, int scaleX, int scaleY) {
		unselect();
		for (int i = contents.size() - 1; i >= 0; i--) {
			Shape s = contents.get(i);
			if (s.isPointInShapesNeighbourhood(p, scaleX, scaleY)) {
				currentlySelected.add(s);
				pictureChanged();
				return;
			}
		}
	}
	
	/**
	 * Unselects all the shapes.  Informs the listeners that
	 * the picture changed.
	 */
	public void unselect() {
		currentlySelected.clear();
		pictureChanged();
	}

	/**
	 * Checks if the Point is in the neighbourhood of any of the selected
	 * shapes.
	 * 
	 * @param p Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return <tt>true</tt> if the Point is in the neighbourhood of any of the
	 * selected shapes
	 */
	public boolean checkIfClickedInSelection(Point2D.Double p, int scaleX, int scaleY) {
		if (currentlySelected.isEmpty()) {
			return false;
		} else {
			for (Shape shape : currentlySelected) {
				if (shape.isPointInShapesNeighbourhood(p, scaleX, scaleY)) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Gets the shape which is near to the Point.  The Point is in
	 * the neighbourhood of the Shape.
	 * 
	 * @param p Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return shape in the neighbourhood
	 */
	public Shape getShapeFromNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		for (int i = 0; i < contents.size(); i++) {
			Shape shape = contents.get(i);
			if (shape.isPointInShapesNeighbourhood(p, scaleX, scaleY)) {
				return shape;
			}
		}
		return null;
	}

	/**
	 * Gets the list of shapes' ids of the currently selected shapes.
	 * 
	 * @return list of shapes' ids of the currently selected shapes
	 */
	public List<Long> getSelectedShapesIdList() {
		List<Long> list = new ArrayList<Long>();
		for (Shape shape : currentlySelected) {
			list.add(shape.getShapeId());
		}
		return list;
	}

	/**
	 * Moves all the currently selected shapes by the Vector.
	 * 
	 * @param vector how we should move the shapes
	 */
	public void moveShapes(Vector vector) {
		for (Shape shape : currentlySelected) {
			shape.changeCoordinates(vector);
		}
		pictureChanged();
	}

	public void selectAll() {
		unselect();
		for (int i = 0; i < contents.size(); i++) {
			currentlySelected.add(contents.get(i));
		}
		pictureChanged();
	}

	/**
	 * Gets the text shape which is in the neighbourhood of the Point. 
	 * 
	 * @param point Point in which neighbourhood should be a shape
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return the clicked text shape<br>
	 * <tt>null</tt> if none text shape was clicked
	 */
	public TextShape getClickedTextShape(Point2D.Double point, int scaleX, int scaleY) {
		for (int i = contents.size() - 1; i >= 0; i--) {
			if (contents.get(i) instanceof TextShape) {
				TextShape shape = (TextShape)contents.get(i);
				if (shape.isPointInShapesInterior(point, scaleX, scaleY)) {
					return shape;
				}
			}
		}
		return null;
	}

	/**
	 * Sets background color.
	 * 
	 * @param newColor new color to be set
	 */
	public void setBackgroundColor(Color newColor) {
		backgroundColor = newColor;
		pictureChanged();
	}
	
	public Color getBackgroundColor () {
		return backgroundColor;
	}

	/**
	 * Gets the most upper left Point of all the shapes on the picture.
	 * 
	 * @return the most upper left Point of all the shapes on the picture
	 */
	public Point2D.Double getTheMostUpperLeftPoint() {
		Point2D.Double point = null;
		for (int i = 0; i < contents.size(); i++) {
			Point2D.Double p = contents.get(i).getTheMostUpperLeftPoint();
			if (point == null) {
				point = new Point2D.Double(p.x, p.y);
			} else {
				if (p.x < point.x) {
					point.x = p.x;
				}
				if (p.y < point.y) {
					point.y = p.y;
				}
			}
		}
		
		return point;
	}
	
	/**
	 * Gets the most upper left Point of the shapes from the shapes' list.  It
	 * adds the margin to the upper left Point.
	 * 
	 * @param list shapes' list for which we calculate the upper left Point
	 * @return the most upper left Point of the shapes from the shapes' list
	 */
	public Point2D.Double getTheMostUpperLeftPointAmongGaveList(List<Shape> list) {
		Point2D.Double point = null;
		for (Shape shape : list) {
			Point2D.Double p = shape.getTheMostUpperLeftPoint();
			if (point == null) {
				point = new Point2D.Double(p.x, p.y);
			} else {
				if (p.x < point.x) {
					point.x = p.x;
				}
				if (p.y < point.y) {
					point.y = p.y;
				}
			}
		}

		point.x -= MARGIN_LENGTH;
		point.y -= MARGIN_LENGTH;
		return point;
	}
	
	/**
	 * Gets the most bottom right Point of the shapes from the shapes' list.
	 * 
	 * @param list shapes' list for which we calculate the bottom right Point
	 * @return the most bottom right Point of the shapes from the shapes' list
	 */
	public Point2D.Double getTheMostBottomRighttPointAmongGaveList(List<Shape> list) {
		Point2D.Double point = null;
		for (Shape shape : list) {
			Point2D.Double p = shape.getTheMostBottomRightPoint();
			if (point == null) {
				point = new Point2D.Double(p.x, p.y);
			} else {
				if (p.x > point.x) {
					point.x = p.x;
				}
				if (p.y > point.y) {
					point.y = p.y;
				}
			}
		}
		return point;
	}
	
	/**
	 * Gets the most bottom right Point of all the shapes on the picture.  It
	 * adds the margin to the bottom right Point.
	 * 
	 * @return the most bottom right Point of all the shapes on the picture
	 */
	public Point2D.Double getTheMostBottomRightPoint() {
		Point2D.Double point = null;
		for (int i = 0; i < contents.size(); i++) {
			Point2D.Double p = contents.get(i).getTheMostBottomRightPoint();
			if (point == null) {
				point = new Point2D.Double(p.x, p.y);
			} else {
				if (p.x > point.x) {
					point.x = p.x;
				}
				if (p.y > point.y) {
					point.y = p.y;
				}
			}
		}
		point.x += MARGIN_LENGTH;
		point.y += MARGIN_LENGTH;
		return point;
	}

	
	public PicturesContent copy() {
		PicturesContent picture = new PicturesContent();
		
		picture.backgroundColor = backgroundColor;
		ShapePriorityStack stack = new ShapePriorityStack();
		for (int i = 0; i < contents.size(); i++) {
			Shape oldS = contents.get(i); 
			Shape s = Plugin.createShape(oldS.getName(), oldS.getShapeId(),
					oldS.getAuthor(), oldS.getCoordinates(), oldS.getColor(),
					oldS.getDisplayHeight(), oldS.isFilled());			
			stack.add(s);
		}
		picture.contents = stack;
		picture.currentlySelected = currentlySelected;
		picture.listeners = listeners;
		
		return picture;
	}
	
}
