package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

/**
 * This class represents the rectangle.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Rectangle extends Shape {
	/**
	 * Tag used to send this shape.
	 */
	public static final String NAME = "rectangle";

	private Point2D.Double upperLeft, bottomRight;

	/**
	 * Constructs a new rectangle and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param a
	 *            shape's author
	 * @param coords
	 *            shape's coordinates converted to String
	 * @param c
	 *            shape's color
	 * @param height
	 *            shape's height in the picture's array
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 */
	public Rectangle(long id, JID a, String coords, Color c, int height,
			boolean filled) {
		super(id, a, c, height);
		setCoordinates(coords);
		isFilled = filled;
	}

	/**
	 * Constructs a new rectangle and sets all the fields.
	 * 
	 * @param p1
	 *            first Point2D.Double determining the rectangle
	 * @param p2
	 *            second Point2D.Double determining the rectangle
	 * @param c
	 *            shape's color
	 * @param a
	 *            shape's author
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 */
	public Rectangle(Point2D.Double p1, Point2D.Double p2, Color c, JID a, boolean filled) {
		super(generateId(), a, c, INCORRECT_HEIGHT);
		upperLeft = PointsOperations.upperLeftPoint(p1, p2);
		bottomRight = PointsOperations.bottomRightPoint(p1, p2);
		isFilled = filled;
	}

	public void draw(Graphics g, int xScale, int yScale) {		
		Point scaledUpperLeft = ceil(upperLeft, xScale, yScale);
		Point scaledBottomRight = ceil(bottomRight, xScale, yScale);
		
		if (isFilled) {
			g.setColor(color);
			g.fillRect(scaledUpperLeft.x + 1, scaledUpperLeft.y + 1,
					scaledBottomRight.x - scaledUpperLeft.x - 1,
					scaledBottomRight.y - scaledUpperLeft.y - 1);
		}
		if (isFilled) {
			g.setColor(borderColor);
		} else {
			g.setColor(color);
		}
		g.drawRect(scaledUpperLeft.x, scaledUpperLeft.y,
				scaledBottomRight.x - scaledUpperLeft.x,
				scaledBottomRight.y - scaledUpperLeft.y);
	}

	public void drawAsSelected(Graphics g, int xScale, int yScale) {
		draw(g, xScale, yScale);
		
		Point scaledUpperLeft = ceil(upperLeft, xScale, yScale);
		Point scaledBottomRight = ceil(bottomRight, xScale, yScale);
		
		g.setColor(selectionColor);
		g.drawRect(scaledUpperLeft.x - 2, scaledUpperLeft.y - 2, 4, 4);
		g.drawRect(scaledBottomRight.x - 2, scaledUpperLeft.y - 2, 4, 4);
		g.drawRect(scaledBottomRight.x - 2, scaledBottomRight.y - 2, 4, 4);
		g.drawRect(scaledUpperLeft.x - 2, scaledBottomRight.y - 2, 4, 4);
	}

	public String getName() {
		return NAME;
	}

	public String getCoordinates() {
		ArrayList<Point2D.Double> points = new ArrayList<Point2D.Double>();
		points.add(upperLeft);
		points.add(bottomRight);
		return PointsOperations.pointsToString(points);
	}

	/**
	 * @param coords
	 *            coordinates in String: first Point2D.Double determining the rectangle,
	 *            second Point2D.Double determining the rectangle
	 */
	public void setCoordinates(String coords) {
		List<Point2D.Double> points = PointsOperations.getPointsFromString(coords);
		upperLeft = PointsOperations.upperLeftPoint(points.get(0), points
				.get(1));
		bottomRight = PointsOperations.bottomRightPoint(points.get(0), points
				.get(1));
	}

	public boolean isInsideTheRect(Point2D.Double externalUpperLeft,
			Point2D.Double externalBottomRight) {

		return (PointsOperations.pointIsInsideRect(upperLeft,
				externalUpperLeft, externalBottomRight) && PointsOperations
				.pointIsInsideRect(bottomRight, externalUpperLeft,
						externalBottomRight));
	}

	public boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		if (isFilled) {
			return isPointInShapesInterior(p, scaleX, scaleY);
		} else {
			return isPointInRectanglesNeighbourhood(p, upperLeft, bottomRight, scaleX, scaleY);
		}
	}

	public Point2D.Double getDisplayPoint() {
		return PointsOperations.getMiddlePoint(upperLeft, new Point2D.Double(
				bottomRight.x, upperLeft.y));
	}

	public void changeCoordinates(Vector v) {
		upperLeft = v.addVectorToPoint(upperLeft);
		bottomRight = v.addVectorToPoint(bottomRight);
	}

	public Point2D.Double getTheMostUpperLeftPoint() {
		return upperLeft;
	}

	public Point2D.Double getTheMostBottomRightPoint() {
		return bottomRight;
	}

	@Override
	public boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY) {
		Point scaledUpperLeft = ceil(upperLeft, scaleX, scaleY);
		Point scaledBottomRight = ceil(bottomRight, scaleX, scaleY);
		Point2D.Double scaledPoint = ceilTo2DPointl(p, scaleX, scaleY);
		
		Point2D.Double surroundedUpperLeft = new Point2D.Double(
				scaledUpperLeft.x + SURROUND_DIST, scaledUpperLeft.y+SURROUND_DIST);
		Point2D.Double surroundedBottomRight = new Point2D.Double(
				scaledBottomRight.x + SURROUND_DIST, scaledBottomRight.y + SURROUND_DIST);
		return PointsOperations.pointIsInsideRect(scaledPoint, surroundedUpperLeft, surroundedBottomRight);
	}
}
