/*
 * Created on 2006-02-25
 *
 */
package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.Plugin;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

/**
 * This class represents an abstract shape.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public abstract class Shape {
	protected final Color DEFAULT_COLOR = Color.black;

	protected final static int INCORRECT_HEIGHT = -1;

	protected Color color;

	protected Color borderColor = Color.BLACK;

	protected Color selectionColor = DEFAULT_COLOR;

	protected long shapeId;

	protected JID author;

	/**
	 * Where the shape should be displayed at the picture (under shapes with
	 * bigger height and above these with smaller).
	 */
	protected int displayHeight;

	protected boolean isFilled = false;

	protected final double SURROUND_DIST = 3;

	/**
	 * Constructs a shape and sets the id, author, color and height.
	 * 
	 * @param id
	 *            shape's id
	 * @param a
	 *            shape's author
	 * @param c
	 *            shape's color
	 * @param height
	 *            shape's height in the picture's array
	 */
	public Shape(long id, JID a, Color c, int height) {
		shapeId = id;
		author = a;
		color = c;
		displayHeight = height;
	}

	/**
	 * Gets shape's color.
	 * 
	 * @return color
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * Generates shape's id.
	 * 
	 * @return shape's id
	 */
	public static long generateId() {
		return Plugin.getRandLong();
	}

	/**
	 * Gets shape's id.
	 * 
	 * @return shape's id
	 */
	public long getShapeId() {
		return shapeId;
	}

	/**
	 * Gets shape's author.
	 * 
	 * @return shape's author
	 */
	public JID getAuthor() {
		return author;
	}

	/**
	 * Checks if the shape is filled.
	 * 
	 * @return <tt>true</tt> if the shape is filled
	 */
	public boolean isFilled() {
		return isFilled;
	}

	/**
	 * Draws the shape on the Graphics.
	 * 
	 * @param g
	 *            where we draw
	 * @param xScale
	 * 			  how it should incerease the shape - x coordinate scale factor
	 * @param yScale
	 * 		  	  how it should incerease the shape - y coordinate scale factor
	 */
	public abstract void draw(Graphics g, int xScale, int yScale);

	/**
	 * Draws the shape on the Graphics as selected.
	 * 
	 * @param g
	 *            where we draw
	 * @param xScale
	 * 			  how it should incerease the shape - x coordinate scale factor
	 * @param yScale
	 * 		  	  how it should incerease the shape - y coordinate scale factor
	 */
	public abstract void drawAsSelected(Graphics g, int xScale, int yScale);

	/**
	 * Gets the shape's name.
	 * 
	 * @return shape's name
	 */
	public abstract String getName();

	/**
	 * Gets shape's coordinates converted to String.
	 */
	public abstract String getCoordinates();

	/**
	 * Sets the coordinates of the shapes given by the String.
	 * 
	 * @param coords
	 *            coordinates given by the String
	 */
	public abstract void setCoordinates(String coords);

	/**
	 * Translates the shape by a given vector.
	 * 
	 * @param v
	 *            vector by which we move the shape
	 */
	public abstract void changeCoordinates(Vector v);

	/**
	 * Sets the shape's color.
	 * 
	 * @param c
	 *            color to be set
	 */
	public void setColor(Color c) {
		color = c;
	}

	/**
	 * Checks if the shape is inside the rectangle. The rectangle is determined
	 * by the upper left and bottom right corner.
	 * 
	 * @param upperLeft
	 *            upper left corner
	 * @param bottomRight
	 *            bottom right corner
	 * @return <tt>true</tt> if the shape is entirely inside the rectangle
	 */
	public abstract boolean isInsideTheRect(Point2D.Double upperLeft, Point2D.Double bottomRight);

	/**
	 * Checks if the Point2D.Double is near to the shape. If we click near the shape we
	 * want to determine whether we clicked on the shape (on its neighbourhood)
	 * or not.
	 * 
	 * @param p
	 *            Point for which we check
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return <tt>true</tt> if the Point is in the shape's neighbourhood
	 */
	public abstract boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY);

	/**
	 * Checks if the Point is near to the line. Line is determined by Points A
	 * and B.
	 * 
	 * @param A
	 *            first Point determining the line
	 * @param B
	 *            second Point determining the line
	 * @param X
	 *            Point for which we check
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return <tt>true</tt> if the Point is in the line's neighbourhood
	 */
	protected boolean isPointInLinesNeighbourhood(Point2D.Double A, Point2D.Double B, Point2D.Double X, int scaleX, int scaleY) {
		Point2D.Double scaledA = ceilTo2DPointl(A, scaleX, scaleY);
		Point2D.Double scaledB =ceilTo2DPointl(B, scaleX, scaleY);
		Point2D.Double scaledX = ceilTo2DPointl(X, scaleX, scaleY);
		
		return (Vector.distanceLinePoint(scaledA, scaledB, scaledX) <= SURROUND_DIST
				&& Vector.isAngleBetweenVectorsLessEqual90(scaledA, scaledB, scaledX)
				&& Vector.isAngleBetweenVectorsLessEqual90(scaledB, scaledA, scaledX)
				|| Vector.distance(scaledA, scaledX) <= SURROUND_DIST
				|| Vector.distance(scaledB, scaledX) <= SURROUND_DIST);
	}

	/**
	 * Determines the Point where we should display a tip who is the author
	 * 
	 * @return where we should display a tip who is the author
	 */
	public abstract Point2D.Double getDisplayPoint();

	protected boolean isPointInRectanglesNeighbourhood(Point2D.Double p,
			Point2D.Double upperLeft, Point2D.Double bottomRight, int scaleX, int scaleY) {
		Point scaledPoint = ceil(p, scaleX, scaleY);
		int x = scaledPoint.x;
		int y = scaledPoint.y;
		
		Point scaledULeft = ceil(upperLeft, scaleX, scaleY);
		Point scaledBRight = ceil(bottomRight, scaleX, scaleY);
		
		return (((x >= scaledULeft.x && x <= scaledBRight.x
				|| Math.abs(scaledULeft.x - x) <= SURROUND_DIST
				|| Math.abs(scaledBRight.x - x) <= SURROUND_DIST)
				&& (Math.abs(scaledULeft.y - y) <= SURROUND_DIST
						|| Math	.abs(scaledBRight.y - y) <= SURROUND_DIST))
						|| ((y >= scaledULeft.y
								&& y <= scaledBRight.y
								|| Math.abs(scaledULeft.y - y) <= SURROUND_DIST
								|| Math.abs(scaledBRight.y - y) <= SURROUND_DIST) 
								&& (Math.abs(scaledULeft.x - x) <= SURROUND_DIST
										|| Math	.abs(scaledBRight.x - x) <= SURROUND_DIST)));
	}

	/**
	 * Checks if the Point is in the shape's interior.
	 * 
	 * @param p
	 *            Point for which we check
	 * @param scaleX TODO
	 * @param scaleY TODO
	 * @return <tt>true</tt> if the Point is inside the shape
	 */
	public abstract  boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY);

	/**
	 * Checks if the shape id higher on the picture than a given shape.
	 * 
	 * @param s
	 *            the shape with which we compare
	 * @return <tt>true</tt> if the shape is higher than a given shape
	 */
	public boolean isHigherThan(Shape s) {
		if (displayHeight > s.displayHeight) {
			return true;
		} else if (displayHeight == s.displayHeight) {
			if (shapeId > s.shapeId) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Gets the display height on the picture.
	 * 
	 * @return the display height
	 */
	public int getDisplayHeight() {
		return displayHeight;
	}

	/**
	 * Sets the display height on the picture.
	 * 
	 * @param i
	 *            height to be set
	 */
	public void setDisplayHeight(int i) {
		displayHeight = i;
	}

	/**
	 * Gets the most upper left point of the shape.
	 * 
	 * @return the most upper left point of the shape
	 */
	public abstract Point2D.Double getTheMostUpperLeftPoint();

	/**
	 * Gets the most bottom right point of the shape.
	 * 
	 * @return the most bottom right point of the shape
	 */
	public abstract Point2D.Double getTheMostBottomRightPoint();

	protected Point2D.Double getTheMostUpperLeftPoint(double[] pointsX, double[] pointsY) {
		Point2D.Double point = null;
		for (int i = 0; i < pointsX.length; i++) {
			if (point == null) {
				point = new Point2D.Double(pointsX[i], pointsY[i]);
			} else {
				if (pointsX[i] < point.x) {
					point.x = pointsX[i];
				}
				if (pointsY[i] < point.y) {
					point.y = pointsY[i];
				}
			}
		}

		return point;
	}

	protected Point2D.Double getTheMostBottomRightPoint(double[] pointsX, double[] pointsY) {
		Point2D.Double point = null;
		for (int i = 0; i < pointsX.length; i++) {
			if (point == null) {
				point = new Point2D.Double(pointsX[i], pointsY[i]);
			} else {
				if (pointsX[i] > point.x) {
					point.x = pointsX[i];
				}
				if (pointsY[i] > point.y) {
					point.y = pointsY[i];
				}
			}
		}

		return point;
	}

	protected int ceil(double a, int scale) {
		return (int) (a*scale);
	}
	
	protected Point ceil(Point2D.Double p, int xScale, int yScale) {
		int x = (int) (p.x * xScale);
		int y = (int) (p.y * yScale);
		return new Point(x, y);
	}
	
	protected Point2D.Double ceilTo2DPointl(Point2D.Double p, int xScale, int yScale) {
		int x = (int) (p.x * xScale);
		int y = (int) (p.y * yScale);
		return new Point2D.Double(x, y);
	}

}
