package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.JTextField;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

/**
 * This class represents the text shape.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class TextShape extends Shape {
	private String text;

	private JTextField textField;

	private Font font;

	private double x, y;

	private double  width, height;

	private boolean isEdited = false;

	private final int TEXT_FIELD_MARGIN = 8;

	/**
	 * Tag used to send this shape.
	 */
	public static final String NAME = "text_shape";

	private static final String UNDERLINE = "_";

	private static final String SPACE = "\\s";

	private static final int BASE_WIDTH = 480;
	
	private static final int BASE_HEIGHT = 360;
	

	/**
	 * Constructs a new text shape and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param a
	 *            shape's author
	 * @param coords
	 *            shape's coordinates converted to String
	 * @param c
	 *            shape's color
	 * @param height
	 *            shape's height in the picture's arrays
	 */
	public TextShape(long id, JID a, String coords, Color c, int height) {
		super(id, a, c, height);
		setCoordinates(coords);
		textField = new JTextField();
	}

	/**
	 * Constructs a new ellipse and sets all the fields.
	 * 
	 * @param author
	 *            shape's author
	 * @param c
	 *            shape's color
	 * @param f
	 *            text's font
	 * @param p
	 *            Point2D.Double determining the location of the text
	 */
	public TextShape(JID author, Color c, Font f, Point2D.Double p) {
		super(generateId(), author, c, INCORRECT_HEIGHT);
		x = p.x;
		y = p.y;
		font = f;
		textField = new JTextField();
		text = "";
	}

	public void draw(Graphics g, int xScale, int yScale) {
		Font scaledFont = setWidthAndHeight(g, xScale, yScale);
		if (isEdited) {
			// JTextField is already displayed by Board
		} else {
			g.setColor(color);
			g.setFont(scaledFont);

			int newX = ceil(x, xScale);
			int newY = ceil(y, yScale);
			int newHeight = ceil(height, yScale);
			
			g.drawString(text, newX, newY + newHeight);
		}
	}

	public void drawAsSelected(Graphics g, int xScale, int yScale) {
		draw(g, xScale, yScale);
		g.setColor(selectionColor);

		int newX = ceil(x, xScale);
		int newY = ceil(y, yScale);
		int newWidth = ceil(width, xScale);
		int newHeight = ceil(height, yScale);

		g.drawRect(newX, newY, newWidth, newHeight);
		g.drawRect(newX - 2, newY - 2, 4, 4);
		g.drawRect(newX + newWidth - 2, newY - 2, 4, 4);
		g.drawRect(newX + newWidth - 2,newY + newHeight - 2, 4, 4);
		g.drawRect(newX - 2, newY + newHeight - 2, 4, 4);
	}
	
	private Font getScaledFont(int xScale, int yScale) {
		int oldSize = font.getSize();
		double factorX = (double)xScale / (double)BASE_WIDTH;
		double factorY = (double)yScale / (double)BASE_HEIGHT;
		
		int newSize = Math.min(ceil(factorX, oldSize), ceil(factorY, oldSize));
		Font scaledFont = new Font(font.getName(), font.getStyle(), newSize);
		return scaledFont;
	}

	private Font setWidthAndHeight(Graphics g, int xScale, int yScale) {
		Font scaledFont = getScaledFont(xScale, yScale);
		Rectangle2D textBounds = getTextBounds(g, scaledFont);
		int notScaledWidth = (int) textBounds.getWidth() + 1;
		int notSacledHeight = (int) textBounds.getHeight() + 1;
		width = (double)notScaledWidth / (double)xScale;
		height = (double)notSacledHeight / (double)yScale;
		return scaledFont;
	}

	private Rectangle2D getTextBounds(Graphics graphics, Font scaledFont) {
		Graphics2D g = (Graphics2D) graphics;
		FontRenderContext frc = g.getFontRenderContext();
		TextLayout layout;
		if (text.equals("")) {
			// we want to know the size of an example string
			layout = new TextLayout("bg", scaledFont, frc);
		} else {
			layout = new TextLayout(text, scaledFont, frc);
		}

		return layout.getBounds();
	}

	private void setTextFieldParameters(Graphics g, int xScale, int yScale) {
		Font scaledFont = setWidthAndHeight(g, xScale, yScale);
		textField.setText(text);
		int emptyChars = 0;
		if (text.equals("")) {
			emptyChars = 70;
		} else {
			emptyChars = 40;
		}

		int newX = ceil(x, xScale);
		int newY = ceil(y, yScale);
		int newWidth = ceil(width, xScale);
		int newHeight = ceil(height, yScale);
		
		textField.setBounds(newX, newY, newWidth + TEXT_FIELD_MARGIN + emptyChars,
				newHeight + TEXT_FIELD_MARGIN);
		textField.setFont(scaledFont);
	}

	/**
	 * Gets the text field with good dimension and text.
	 * 
	 * @param g
	 *            Graphics on which we will determine the dimension of the text
	 *            field
	 * @return the set text field
	 */
	public JTextField getTextField(Graphics g, int xScale, int yScale) {
		setTextFieldParameters(g, xScale, yScale);
		return textField;
	}

	/**
	 * Starts edition of the text.
	 */
	public void startEditing() {
		isEdited = true;
		textField.setVisible(true);
	}

	/**
	 * Stops edition of the text.
	 */
	public void stopEditing() {
		isEdited = false;
		text = textField.getText();
		textField.setVisible(false);
	}

	/**
	 * Checks if the text of the TextShape is an empty String.
	 * 
	 * @return <tt>true</tt> if the text is an empty String
	 */
	public boolean isEmptyText() {
		return text.equals("");
	}

	public String getName() {
		return NAME;
	}

	public String getCoordinates() {
		return x + " " + y + " "
				+ changeRegexInString(font.getFamily(), SPACE, UNDERLINE) + " "
				+ font.getStyle() + " " + font.getSize() + " " + text;
	}

	/**
	 * Changes regexFrom into regexTo in string. For example:
	 * changeRegexInString("I am here", "\\s", "_") returns "I_am_here"
	 * 
	 * @param string
	 * @param regexFrom
	 *            what we want to replace
	 * @param regexTo
	 *            what we want to get in the place of regexFrom
	 * @return changed string
	 */
	private String changeRegexInString(String string, String regexFrom,
			String regexTo) {
		String[] parts = string.split(regexFrom);
		String res = "";
		for (int i = 0; i < parts.length; i++) {
			res = res + parts[i] + regexTo;
		}
		return res;
	}

	/**
	 * @param coords
	 *            coordinates in String: lacation of the text: x and y, text's
	 *            font with no space chars, text's style, text's size, text
	 */
	public void setCoordinates(String coords) {
		String[] result = coords.split(SPACE);
		x = Double.parseDouble(result[0]);
		y = Double.parseDouble(result[1]);
		font = new Font(changeRegexInString(result[2], UNDERLINE, SPACE),
				Integer.parseInt(result[3]), Integer.parseInt(result[4]));
		text = "";
		for (int i = 5; i < result.length; i++) {
			text += result[i] + " ";
		}
	}

	public void changeCoordinates(Vector v) {
		Point2D.Double p = v.addVectorToPoint(new Point2D.Double(x, y));
		x = p.x;
		y = p.y;
	}

	public boolean isInsideTheRect(Point2D.Double upperLeft, Point2D.Double bottomRight) {
		return PointsOperations.pointIsInsideRect(new Point2D.Double(x, y), upperLeft,
				bottomRight)
				&& PointsOperations.pointIsInsideRect(new Point2D.Double(x + width, y
						+ height), upperLeft, bottomRight);
	}

	public boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		return (isPointInShapesInterior(p, scaleX, scaleY) ||
			isPointInRectanglesNeighbourhood(p, new Point2D.Double(x, y), new Point2D.Double(x
				+ width, y + height), scaleX, scaleY));
	}

	public boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY) {
		return PointsOperations.pointIsInsideRect(p, new Point2D.Double(x, y),
				new Point2D.Double(x + width, y + height));
	}

	public Point2D.Double getDisplayPoint() {
		return PointsOperations.getMiddlePoint(new Point2D.Double(x, y), new Point2D.Double(x
				+ width, y));
	}

	public Point2D.Double getTheMostUpperLeftPoint() {
		return new Point2D.Double(x, y);
	}

	public Point2D.Double getTheMostBottomRightPoint() {
		return new Point2D.Double(x + width, y + height);
	}

}
