package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents an abstract action.  Action is an operation that was
 * made by a user and that changed the picture. It is sent to all the other
 * users. After executing all actions we can obtain the history of the picture.
 * After creating the action it should be executed to make a difference on
 * the picture.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public abstract class Action {
	protected List<Long> shapeIdList;
	protected Map<Long, Shape> allShapes;
	protected String time;
	protected String actionsAuthor;
	protected int counter;
	
	/**
	 * Creates a new action and sets all the required fields.
	 * 
	 * @param sIdList list of shapes' ids that are changed by this action
	 * @param allS map of all shapes that have ever been created
	 * @param t action's time
	 * @param author Action's author
	 * @param nr action's counter
	 */
	public Action(List<Long> sIdList, Map<Long, Shape> allS,
			String t, String author, int nr) {
		shapeIdList = sIdList;
		allShapes = allS;
		time = t;
		actionsAuthor = author;
		counter = nr;
	}
	
	/**
	 * Makes the changes to the current picture.  It executes the actual
	 * function of the action. 
	 *  
	 * @param shapesBoard where it should make changes
	 */
	public abstract void execute(PicturesContent shapesBoard);
	
	/**
	 * Returns the action required to process undo and redo action. Gives 
	 * the oposite effect to the Action.
	 * 
	 * @return action required to process undo and redo action. Gives 
	 * the oposite effect to the Action
	 */
	public abstract Action giveComplementaryAction(String time, int counter);
	
	/**
	 * Creates the messages that will be sent to other users.  A message
	 * contains all the informations required to create an Action. It creates
	 * messages to all the receivers.
	 * 
	 * @param receivers receivers' list
	 * @param thread chat's thread
	 * @return messages messages describing the Action addressed to all
	 * teh receivers 
	 */
	public abstract List<PictureMessage> getPictureMessageList(
			List<JID> receivers, String thread);

	/**
	 * Creates an one-element shape's ids' list containing the given element.
	 * 
	 * @param elem shape's id that will be in the list 
	 * @return one-element list containing the given shape's id 
	 */
	public static List<Long> createShapeIdList(long elem) {
		List<Long> shapeIdList = new ArrayList<Long>();
		shapeIdList.add(elem);
		return shapeIdList;
	}
	
	/**
	 * Creates an one-element shapes' list containing the given element.
	 * 
	 * @param elem shape that will be in the list 
	 * @return one-element list containing the given shape
	 */
//	public static List<Shape> createShapeList(Shape elem) {
//		List<Shape> shapeList = new ArrayList<Shape>();
//		shapeList.add(elem);
//		return shapeList;
//	}

//	protected List<Shape> getShapeListFromIdList(List<Long> idList) {
//		if (idList != null) {
//			List<Shape> list = new ArrayList<Shape>();
//			for (Long id : idList) {			
//				list.add(allShapes.get(id));
//			}
//			return list;
//		} else {
//			return null;
//		}
//	}
	
	protected static List<Long> getShapeIdList(List<Shape> shapes) {
		List<Long> list = new ArrayList<Long>();
		for (Shape shape : shapes) {
			list.add(shape.getShapeId());
		}
		return list;
	}
	
	/**
	 * Checks if the Action happened before the given Action.
	 * 
	 * @param a Action with which we compare the Action
	 * @return <tt>true</tt> if the Action happened before the given Action
	 */
	public boolean happenedBefore(Action a) {
		if (counter < a.counter) {
			return true;
		} else if (counter > a.counter) {
			return false;
		} else {
			// equal numbers
			return (actionsAuthor.compareTo(a.actionsAuthor) < 0);
		}
	}

	/**
	 * Gets Action's counter.
	 * 
	 * @return Action's counter
	 */
	public int getCounter() {
		return counter;
	}

	/**
	 * Gets Action's time.
	 * 
	 * @return Action's time
	 */
	public String getTime() {
		return time;
	}
	
	/**
	 * Gets the number of the shapes that are changed by the Action.
	 * 
	 * @return number of the shapes that are changed by the Action
	 */
	public int getShapesNumber() {
		if (shapeIdList != null) {
			return shapeIdList.size();
		} else {
			return -1;
		}
	}
	
	/**
	 * Sets Action's counter.
	 * 
	 * @param c new counter
	 */
	public void setCounter(int c) {
		counter = c;
	}
}
