package nu.fw.jeti.plugins.drawing.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import nu.fw.jeti.backend.roster.NormalJIDStatus;
import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.util.I18N;

/**
 * This class implements a JDialog to add users to the chat.  It displays 
 * a tree of users, that are not yet in the chat. The user can choose some
 * nicks and add them to the chat.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class AddUsersDialog extends JDialog {
	private Set<UserRepresentation> selectedUsers;
	
	private static final long serialVersionUID = -609695871275625226L;

	/**
	 * Creates a dialog component.  This component is modal.
	 * @param pictureChat where dialog should send the results
	 * @param backend chat's backend
	 * @param title TODO
	 * @param selectCurrentUsers TODO
	 */
	public AddUsersDialog(UsersSelectAction action, PictureChat pictureChat,
			Backend backend, String title, boolean selectCurrentUsers) {
		setTitle(title);
		selectedUsers = new HashSet<UserRepresentation>();
		JScrollPane scrollPane = createScrollPane(backend, pictureChat, selectCurrentUsers);
		JButton addButton = createAddButton(action);
		JButton cancelButton = createCancelButton();
		getContentPane().setLayout(new BorderLayout());
		
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
		p.add(addButton);
		p.add(cancelButton);
		getContentPane().add(p, BorderLayout.SOUTH);
		pack();
		setModal(true);
		setLocationRelativeTo(null);
	}

	private JScrollPane createScrollPane(Backend backend, PictureChat chat,
			boolean selectCurrentUsers) {
		String[] groups = backend.getAllGroups();

		List<UserRepresentation> users = new ArrayList<UserRepresentation>();
		Iterator iter = Roster.getJIDStatussen();
		for (Iterator iterator = iter; iterator.hasNext();) {
			NormalJIDStatus jidStatus = (NormalJIDStatus) iterator.next();
			if (selectCurrentUsers) {
				if (!chat.thereIsNoSuchUserYet(jidStatus.getJID())) {
					users.add(new UserRepresentation(jidStatus));
				}
			} else { 
				if (chat.thereIsNoSuchUserYet(jidStatus.getJID())) {
					users.add(new UserRepresentation(jidStatus));
				}
			}
		}
		
		JTree tree;
	    DefaultMutableTreeNode top =
	        new DefaultMutableTreeNode(I18N.getText("drawing" ,"Groups"));
	    Vector<DefaultMutableTreeNode> groupNodeVector = new Vector<DefaultMutableTreeNode>();
	    // creates group nodes
	    for (int i = 0; i < groups.length; i++) {
	    	DefaultMutableTreeNode groupNode = new DefaultMutableTreeNode(groups[i]);
	    	top.add(groupNode);
			groupNodeVector.add(groupNode);
			
		}
	    
	    // creates user nodes - each in a group
	    for (DefaultMutableTreeNode node : groupNodeVector) {
			for (UserRepresentation user : users) {
				if (user.isInThisGroup(node.toString())) {
					node.add(new DefaultMutableTreeNode(user));
				}
			}
		}

		if (selectCurrentUsers) {
			UserRepresentation me = new UserRepresentation(chat.getMyJID());
			DefaultMutableTreeNode myNode = new DefaultMutableTreeNode(me);
			top.add(myNode);
		}
	    
	    tree = new JTree(top);
	    tree.setCellRenderer(new IconStatusRenderer());
	    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
	    tree.addTreeSelectionListener(new UsersSelectionListener(tree));
		
		JScrollPane pane = new JScrollPane(tree);
		pane.setPreferredSize(new Dimension(180,200));
		return pane;
	}

	private JButton createAddButton(UsersSelectAction action) {
		JButton addButton = new JButton(I18N.getText("drawing","Add"));
		addButton.addActionListener(new AddUsersListener(action));
		return addButton;
	}
	
	private JButton createCancelButton() {
		JButton cancelButton = new JButton(I18N.getText("drawing","Cancel"));
		cancelButton.addActionListener(new CancelListener());
		return cancelButton;
	}
	
	class AddUsersListener implements ActionListener {
		UsersSelectAction action;

		public AddUsersListener(UsersSelectAction a) {
			action = a;
		}

		public void actionPerformed(ActionEvent e) {
			action.obsluzSelectedUsers(selectedUsers);
			dispose();
		}
		
	}
	
	class CancelListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			dispose();
		}
		
	}
	
	class IconStatusRenderer extends DefaultTreeCellRenderer {
		private static final long serialVersionUID = -1848585234057535801L;
		
	    public Component getTreeCellRendererComponent(
                JTree tree,
                Object value,
                boolean sel,
                boolean expanded,
                boolean leaf,
                int row,
                boolean hasFocus) {
			
			super.getTreeCellRendererComponent(
			                tree, value, sel,
			                expanded, leaf, row,
			                hasFocus);
			if (leaf) {
				// user respresentation or an empty group
				DefaultMutableTreeNode node =
	                (DefaultMutableTreeNode)value;
				if (node.getUserObject() instanceof UserRepresentation) {
					UserRepresentation user =
		                (UserRepresentation)(node.getUserObject());
					setIcon(user.getStatusIcon());
					setToolTipText(user.getJIDAddress());
				}				
			}
			
			return this;
		}
	}
	
	class UsersSelectionListener implements TreeSelectionListener {
		JTree tree;
		
		public UsersSelectionListener(JTree t) {
			tree = t;
		}

		public void valueChanged(TreeSelectionEvent e) {
			TreePath[] paths = e.getPaths();
			for (TreePath path : paths) {
				Object obj = ((DefaultMutableTreeNode)path.
					getLastPathComponent()).getUserObject();
				if (e.isAddedPath(path)) {
					// we added it; we should add it to the selection items					
					if (obj instanceof UserRepresentation) {
						selectedUsers.add((UserRepresentation)obj);
					} 
//					else if (obj instanceof String) { // group or root selected
//						// nie wiem, jak zaznaczac cala galaz	
//					}
				} else {
					if (obj instanceof UserRepresentation) {
						selectedUsers.remove((UserRepresentation)obj);
					}
				}
			}
		}
		
	}
}
