package nu.fw.jeti.plugins.drawing.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;

import javax.swing.*;

import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents the board of the picture - the panel where all
 * the shapes are displayed.  It repaints when the listener receives
 * the information that the picture has been changed. 
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Board extends JPanel {
	
	private PictureChat chat;
	private Color currentColor = Color.BLACK;
	private Color selectionColor = Color.BLACK;
	private final Color SELECTION_BACKGROUND_COLOR = new Color(153, 0, 153, 35); 
	private Shape currentShape = null;
	private int currentTransparency = TransparencySettingAction.TRANS_MAX;
	private PicturesContent picture;
	
	private Mode currentMode;
	private String fontFamily = "Serif";
	private int fontSize = 12;
	private int fontStyle = Font.PLAIN;
	
	private static final long serialVersionUID = 1L;
	public static final Color DEFAULT_BACKGROUND_COLOR = Color.WHITE;
	private Color currentBackgroundColor = DEFAULT_BACKGROUND_COLOR;
	private Color fontForegroundColor = Color.BLACK;
	private Color fontBackgroundColor = DEFAULT_BACKGROUND_COLOR;
	
	/**
	 * Creates an empty board and set the background to
	 * the DEFAULT_BACKGROUND_COLOR.
	 * 
	 * @param pictureChat chat which give the board informations about authors
	 * and JIDs.
	 * @param p all the informations about the picture; board tells picture
	 * to draw all the shapes.
	 */
	public Board(PictureChat pictureChat, PicturesContent p) {
		super();
		chat = pictureChat;
		picture = p;
		addMouseListener(new MouseL());
		addMouseMotionListener(new MouseMotionL());
		picture.addListener(new PictureChangeListener());
		
		setFocusable(true); // it gets focus even after pushing a button
		setBackground(currentBackgroundColor);
		
		ToolTipManager.sharedInstance().registerComponent(this);
		
		setLayout(null);
	}

	/**
	 * Sets current mode according to the user's choose and repaints.
	 * 
	 * @param m new mode
	 */
	public void setCurrentMode(Mode m) {
		if (currentMode != null) {
			currentMode.dispose();
		}
		currentMode = m;
		repaint();
	}
	
	protected void setCurrentShape(Shape s) {
		currentShape = s;
	}
	
	protected Shape getCurrentShape() {
		return currentShape;
	}
	
	protected Color getSelectionColor() {
		return selectionColor;
	}
	
	public Color getCurrentColor() {
		return currentColor;
	}
	
	/**
	 * Paints the component with the shapes that should be actually on
	 * the picture.
	 * 
	 * @param g where it should draw
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Dimension size = getSize();
		picture.drawPictureWithSelection(g, size.width, size.height);
		if (currentShape != null) {
			currentShape.draw(g, size.width, size.height);
		}
	}
	
	/**
	 * Sets current drawing color.
	 * 
	 * @param c new color
	 */
	public void setCurrentColor(Color c) {
		currentColor = c;
		setFontForegroundColor(c);
		requestFocusInWindow();
	}
	
	/**
	 * Creates a tool tip with a name of an author of the shape that was
	 * currently pointed out.
	 * 
	 * @param event stores the Point where is a shape, that the user wants
	 * to get info about 
	 */
	public String getToolTipText(MouseEvent event) {
		Point2D.Double currentPoint = scaleToZeroOne(event.getPoint());
		Dimension size = getSize();
		JID jid = chat.getAuthor(currentPoint, size.width, size.height);
		if (jid != null) {
			UserRepresentation userRep = new UserRepresentation(jid);
			return (userRep.toString());
		} else {
			return null;
		}
    }
	
	/**
	 * Adds a text field to the board.
	 * 
	 * @param textField text field to be added
	 */
	public void addTextField(JTextField textField) {
		add(textField);
		textField.grabFocus();
		textField.setVisible(true);
	}

	protected void showPopupWithAuthor(String author, int x, int y) {
		PopupFactory popupFactory = PopupFactory.getSharedInstance();
		JToolTip tip = new JToolTip();
		tip.setTipText(author);
		Popup popup = popupFactory.getPopup(this, tip, x, y);
		popup.show();
		Timer timer = new Timer(5000, new HidePopupManagement(popup));
		timer.setRepeats(false);
		timer.start();
	}
	
	class MouseL implements MouseListener {

		public void mouseClicked(MouseEvent e) {
			Point2D.Double point = scaleToZeroOne(e.getPoint());
			Dimension size = getSize();
			currentMode.mouseClicked(e, currentColor, point, size.width, size.height);
		}

		public void mousePressed(MouseEvent e) {
			Point2D.Double point = scaleToZeroOne(e.getPoint());
			Dimension size = getSize();
			currentMode.mousePressed(point, currentColor, size.width, size.height);
		}

		public void mouseReleased(MouseEvent e) {
			Point2D.Double point = scaleToZeroOne(e.getPoint());
			Dimension size = getSize();
			currentMode.mouseReleased(point, size.width, size.height);
		}

		public void mouseEntered(MouseEvent e) {}

		public void mouseExited(MouseEvent e) {}
		
	}
	
	class MouseMotionL implements MouseMotionListener {

		public void mouseDragged(MouseEvent e) {
			Point2D.Double point = scaleToZeroOne(e.getPoint());
			currentMode.mouseDragged(point);			
		}

		public void mouseMoved(MouseEvent e) {
			Point2D.Double point = scaleToZeroOne(e.getPoint());
			currentMode.mouseMoved(point);
		}
		
	}
	
	/**
	 * If the picture is changed, it sends a signal to the listener 
	 * and the listener draws the board.
	 * 
	 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
	 *
	 */
	class PictureChangeListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			chat.showWindow();
			if (e != null) {
				switch (e.getID()) {
				case PicturesContent.AUTHOR_DISPLAY:
					displayAuthor(e);
					break;

				default:
					break;
				}
			}
//			invalidate();
//			Dimension size = getSize();
			repaint();
		}

		private void displayAuthor(ActionEvent e) {
			String[] result = e.getActionCommand().split("\\s");
			String author = result[0];
			if (!author.equals(chat.getMyJID().toString())) {
				// I don't display my id
				double notScaledX = Double.parseDouble(result[1]);
				double notScaledY = Double.parseDouble(result[2]);
				Point scaledPoint = scaleToNormalSize(
						new Point2D.Double(notScaledX, notScaledY));
				
				int x = scaledPoint.x + getLocationOnScreen().x;
				int y = scaledPoint.y + getLocationOnScreen().y;
				
				JID jid = null;
				try {
					jid = JID.checkedJIDFromString(author);
				} catch (InstantiationException e1) {
					e1.printStackTrace();
				}
				
				JIDStatus jidStatus = Roster.getJIDStatus(jid);
				String nick;
				if (jidStatus == null) {
					// user is not in our contact list
					nick = author;
				} else {
					nick = jidStatus.getNick();
				}
				showPopupWithAuthor(nick, x, y);
			}
		}

		private Point scaleToNormalSize(Point2D.Double point) {
			Dimension size = getSize();
			int x = (int)(point.x * size.width);
			int y = (int)(point.y * size.height);
			return new Point(x, y);
		}
		
	}
	
	class HidePopupManagement implements ActionListener {
		Popup popup;
		public HidePopupManagement(Popup p) {
			popup = p;
		}
		
		public void actionPerformed(ActionEvent e) {
			popup.hide();
		}		
	}

	/**
	 * Gets current font.
	 * 
	 * @return current font
	 */
	public Font getCurrentFont() {
		return new Font(fontFamily, fontStyle, fontSize);
	}

	/**
	 * Sets font's family.
	 * 
	 * @param family font's family
	 */
	public void setFontFamily(String family) {
		fontFamily = family;
	}	

	/**
	 * Gets font's background color.
	 * 
	 * @return font's background color
	 */
	public Color getFontBackgroundColor() {
		return fontBackgroundColor;
	}

	/**
	 * Sets font's background color.
	 * 
	 * @param fontBackColor new font's background color
	 */
	public void setFontBackgroundColor(Color fontBackColor) {
		fontBackgroundColor = fontBackColor;
	}

	/**
	 * Gets font's foreground color.
	 * 
	 * @return font's foreground color
	 */
	public Color getFontForegroundColor() {
		return fontForegroundColor;
	}

	/**
	 * Sets font's foreground color.
	 * 
	 * @param fontForeColor new font's foreground color
	 */
	public void setFontForegroundColor(Color fontForeColor) {
		fontForegroundColor = fontForeColor;
	}

	/**
	 * Sets font's size.
	 * 
	 * @param size font's size
	 */
	public void setFontSize(int size) {
		fontSize = size;
	}

	/**
	 * Sets font's style.
	 * 
	 * @param style font's style
	 */
	public void setFontStyle(int style) {
		fontStyle = style;
	}
	
	/**
	 * Sets transparency of the colors that will be drawn from now.
	 * 
	 * @param alpha alpha value in the range 0-255
	 */
	public void setCurrentTransparency(int alpha) {
		currentTransparency = alpha;
		Color c = new Color(currentColor.getRed(), currentColor.getGreen(),
				currentColor.getBlue(), alpha);
		setCurrentColor(c);
	}
	
	/**
	 * Returns the alpha component in the range 0-255.
	 * 
	 * @return alpha component
	 */
	public int getCurrentTransparency() {
		return currentTransparency;
	}

	/**
	 * Gets the clor that is used to display selection.  The selection is shown
	 * as a filled rectangle.
	 * 
	 * @return background color of selection
	 */
	public Color getSelectionBackgroundColor() {
		return SELECTION_BACKGROUND_COLOR;
	}

	/**
	 * Gets current background color.
	 * 
	 * @return current background color
	 */
	public Color getCurrentBackgroundColor() {
		return currentBackgroundColor;
	}

	/**
	 * Sets current background color and sets the background.
	 * 
	 * @param bc new background color
	 */
	public void setCurrentBackgroundColor(Color bc) {
		currentBackgroundColor = bc;
		setBackground(currentBackgroundColor);
	}
	
	public Point2D.Double scaleToZeroOne(Point p) {
		Dimension size = getSize();
		double x = (double)p.x / (double)size.width;
		double y = (double)p.y / (double)size.height;
		return new Point2D.Double(x, y);
	}
	
}

