package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.shapes.Ellipse;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents a mode that draws an
 * {@link nu.fw.jeti.jabber.elements.drawing.Ellipse}.  A user presses
 * the mouse and this Point becomes an upper left corner of the rectangle
 * in which there will be an ellipse. Then he drags the mouse and chooses
 * the dimension of the ellipse. When he releases the mouse the ellipse is
 * drawn, memorized in the history and sent to the other users.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class EllipseMode extends Mode {
	private boolean isFilled;

	/**
	 * Creates an ellipse mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 * @param filled <tt>true</tt> if the ellipse is filled
	 */
	public EllipseMode(PictureChat c, Board b, String text, 
			ImageIcon i, String desc, Integer m, boolean filled) {
		super(c, b, text, i, desc, m);
		isFilled = filled;
	}

	/**
	 * The Point where a user clicked becomes an upper left corner
	 * of the rectangle in which there will be an ellipse. 
	 */
	public void mousePressed(Double point, Color c, int scaleX, int scaleY) {
		super.mousePressed(point, c, scaleX, scaleY);
		chat.unselect();
	}

	/**
	 * After releasing the mouse the ellipse is memorized in the history, drawn
	 * and sent to the other users.
	 */
	public void mouseReleased(Double point, int scaleX, int scaleY) {
		Point2D.Double upperLeft = PointsOperations.upperLeftPoint(start, point);
		Point2D.Double bottomRight = PointsOperations.bottomRightPoint(start, point);
		double width = bottomRight.x - upperLeft.x;
		double height = bottomRight.y - upperLeft.y;
		
		Shape shape = new Ellipse(upperLeft.x, upperLeft.y, 
				width, height, color, chat.getMyJID(), isFilled);
		chat.draw(shape);
		board.setCurrentShape(null);
	}

	/**
	 * When the user is dragging the mouse the ellipse grows or becomes
	 * smaller.  The ellipse is shown on the picture.
	 */
	public void mouseDragged(Double point) {
		Point2D.Double upperLeft = PointsOperations.upperLeftPoint(start, point);
		Point2D.Double bottomRight = PointsOperations.bottomRightPoint(start, point);
		double width = bottomRight.x - upperLeft.x;
		double height = bottomRight.y - upperLeft.y;
		
		if (board.getCurrentShape() != null) {
			(board.getCurrentShape()).setCoordinates(
							upperLeft.x + " " + upperLeft.y + " " + width + " " + height);
		} else {
			board.setCurrentShape(new Ellipse(upperLeft.x, upperLeft.y, 
					width, height, color, chat.getMyJID(), isFilled));
		}
		board.repaint();		
	}

}
