package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.util.Vector;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Polygon;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents a mode that draws a filled polyline, which is really
 * a filled {@link nu.fw.jeti.jabber.elements.drawing.Polygon}.  A user presses
 * the mouse and starts drawing the polyline. When he drags the mouse, all
 * the points he drags become the vertices of the polygon. When he releases
 * the mouse the polygon is memorized in the history and sent to the other
 * users. This is a different mode from filled
 * {@link nu.fw.jeti.ui.drawing.PolygonMode}, because it takes all the points
 * which user shows, this is drawn like in
 * {@link nu.fw.jeti.ui.drawing.PolylineMode}. Although this is drawn as filled
 * rectangle.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class FilledPolylineMode extends Mode {
	private Vector<Point2D.Double> points = new Vector<Point2D.Double>();
	private boolean isFilled = true;

	/**
	 * Creates a filled polyline mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 */
	public FilledPolylineMode(PictureChat c, Board b, String text, 
			ImageIcon i, String desc, Integer m) {
		super(c, b, text, i, desc, m);
		points = new Vector<Point2D.Double>();
	}

	public void dispose() {
		super.dispose();
		points.clear();
	}

	/**
	 * Starts drawing the polyline.
	 */
	public void mousePressed(Double point, Color c, int scaleX, int scaleY) {
		color = c;
		points.add(point);
		chat.unselect();
	}

	/**
	 * Stops drawing the polyline, memorizes the shapes in the history, draws
	 * it and sends it to other users. 
	 */
	public void mouseReleased(Double point, int scaleX, int scaleY) {	
		Shape shape = new Polygon(points, color, chat.getMyJID(), isFilled);
		chat.draw(shape);
		points.clear();
		board.setCurrentShape(null);
	}

	/**
	 * The Point from the event becomes the next point in the polygon.
	 */
	public void mouseDragged(Double point) {
		points.add(point);
		if (board.getCurrentShape() != null) {
			(board.getCurrentShape()).setCoordinates(PointsOperations.pointsToString(points));
		} else {
			board.setCurrentShape(new Polygon(points, color, chat.getMyJID(), isFilled));
		}
		board.repaint();
	}

}

