package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.geom.Point2D;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Rectangle;

/**
 * This class represents a mode that allows user to select shapes.  A user
 * presses the mouse and this Point becomes a first point of the rectangle
 * that shows the selected area. Then he drags the mouse and chooses
 * the area of the selection. When he releases the mouse all the shapes that
 * were inside the rectangle, appear as selected. If the user clicks the mouse,
 * it selects a shape that is the newest and in the neighbourhood of the point.
 * If there is none shapes selected, none are shown as selected.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class SelectionMode extends Mode {
	private boolean isFilled = true;
	/**
	 * Determines if we are moving the selected shapes or we are selecting them.
	 */
	private boolean isMoving = false;

	/**
	 * Creates a selection mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 */
	public SelectionMode(PictureChat c, Board b, String text, ImageIcon i, String desc, Integer m) {
		super(c, b, text, i, desc, m);
	}
	
	public void dispose() {
		super.dispose();
		isMoving = false;
	}

	/**
	 * The pressed Point becomes a first point of the rectangle
	 * that shows the selected area.
	 */
	public void mousePressed(Point2D.Double point, Color c, int scaleX, int scaleY) {
		super.mousePressed(point, c, scaleX, scaleY);
		color = board.getSelectionBackgroundColor();
		isMoving = chat.checkIfClickedInSelection(point, scaleX, scaleY);
		end = start;
	}
	
	/**
	 * Selects shown shapes.  If the user was movingthe mouse, all the shapes
	 * that were inside the rectangle, appear as selected. If the user only
	 * clicked, it selects a shape that is the newest and in the neighbourhood
	 * of the point. If there is none shapes selected, none are shown as
	 * selected.
	 */
	public void mouseReleased(Point2D.Double point, int scaleX, int scaleY) {
		if (isMoving) {
			chat.moveShapes(new Vector(end, point));
			end = point;
			Vector transVector = new Vector(start, end);
			chat.moveShapes(transVector.getReverseVector());
			chat.sendShapesTranslation(transVector);
		} else {
			end = point;
			if (start.x == end.x && start.y == end.y) {
				// we want to select only one shape
				chat.selectItem(start, scaleX, scaleY);
			} else {
				chat.selectItems(start, end);
			}
		}
		board.setCurrentShape(null);
	}

	/**
	 * The user chooses the area of the selection.
	 */
	public void mouseDragged(Point2D.Double point) {
		if (isMoving) {
			chat.moveShapes(new Vector(end, point));
			end = point;
		} else {
			end = point;
			if (board.getCurrentShape() != null) {
				(board.getCurrentShape()).setCoordinates(
						PointsOperations.pointsToString(start, end));
			} else {
				board.setCurrentShape(new Rectangle(
						start, end, color, chat.getMyJID(), isFilled));
			}
			board.repaint();	
		}	
	}

}
