package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D.Double;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.TextShape;

/**
 * This class represents a mode that allows user to write.  A user clicks
 * the mouse and this Point becomes an upper left corner of the writing area.
 * The text field appears and the user can type the chars, copy them and paste.
 * When he presses Enter of clicks on the other part of the picture,
 * the text field disappears and a normal written text appears.
 * If the user double-clicks on the text, the text field with the previous text
 * appears and the user can edit it.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class WritingMode extends Mode {

	/**
	 * Creates a writing mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 */
	public WritingMode(PictureChat c, Board b, String text, 
			ImageIcon i, String desc, Integer m) {
		super(c, b, text, i, desc, m);
	}

	public void dispose() {
		if (board.getCurrentShape() instanceof TextShape) {
			stopEditing();
		}
		super.dispose();
	}
	
	/**
	 * The clicked point becomes an upper left corner of the writing
	 * area.  After double-click it starts editing the text near to the point.
	 * If the click point was outside the writing area, it stops editing
	 * the previous text.
	 */
	public void mouseClicked(MouseEvent e, Color c, Double point, int scaleX, int scaleY) {
		chat.unselect();
		if (e.getClickCount() >= 2) {
			TextShape shape = chat.getClickedTextShape(point, scaleX, scaleY);
			if (shape != null) {
				board.setCurrentShape(shape);
				startEditing(shape, scaleX, scaleY);
				return;
			}
		}
		if (board.getCurrentShape() == null) {
			TextShape s = new TextShape(
					chat.getMyJID(), board.getFontForegroundColor(),
					board.getCurrentFont(), point);
			startEditing(s, scaleX, scaleY);
		} else {
			// we look if it is clicked in out shape; if not, we have to finish editing the shape
			if (board.getCurrentShape() instanceof TextShape) {
				if (!board.getCurrentShape().isPointInShapesInterior(point, scaleX, scaleY)) {
					stopEditing();
				}
			}					
		}
		board.repaint();
	}
	
	private void startEditing(TextShape shape, int scaleX, int scaleY) {
		shape.startEditing();
		shape.getTextField(board.getGraphics(), scaleX, scaleY).addActionListener(new EnterListener());
		board.addTextField(shape.getTextField(board.getGraphics(), scaleX, scaleY));
		board.setCurrentShape(shape);
	}

	private void stopEditing() {
		if (board.getCurrentShape() != null) {
			((TextShape)board.getCurrentShape()).stopEditing();
			if (!((TextShape)board.getCurrentShape()).isEmptyText()) {
				chat.draw(board.getCurrentShape());
			}
			board.setCurrentShape(null);
		}
	}
	
	class EnterListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			stopEditing();
		}
		
	}

}
