/**
 * $RCSfile: RTPBridge.java,v $
 * $Revision: 1.1 $
 * $Date: 2007/07/15 15:20:41 $
 *
 * Copyright 2003-2005 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nu.fw.jeti.plugins.jingle.jingle.nat;

import java.util.Iterator;
import java.util.Observer;

import nu.fw.jeti.events.DiscoveryListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.DiscoveryInfo;
import nu.fw.jeti.jabber.elements.DiscoveryItem;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.IQExtension;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;

import org.xml.sax.Attributes;

/**
 * RTPBridge IQ Packet used to request and retrieve a RTPBridge Candidates that can be used for a Jingle Media Transmission between two parties that are behind NAT.
 * This Jingle Bridge has all the needed information to establish a full UDP Channel (Send and Receive) between two parties.
 * <i>This transport method should be used only if other transport methods are not allowed. Or if you want a more reliable transport.</i>
 * <p/>
 * High Level Usage Example:
 * <p/>
 * RTPBridge rtpBridge = RTPBridge.getRTPBridge(xmppConnection, sessionID);
 *
 * @author Thiago Camargo
 */
public class RTPBridge extends Extension implements IQExtension {

    private String sid;
    private String pass;
    private String ip;
    private String name;
    private int portA = -1;
    private int portB = -1;
    private String hostA;
    private String hostB;
    private BridgeAction bridgeAction = BridgeAction.create;

    private enum BridgeAction {

        create, change, publicip
    }

    /**
     * Element name of the packet extension.
     */
    public static final String NAME = "rtpbridge";

    /**
     * Element name of the packet extension.
     */
    public static final String ELEMENT_NAME = "rtpbridge";

    /**
     * Namespace of the packet extension.
     */
    public static final String NAMESPACE = "http://www.jivesoftware.com/protocol/rtpbridge";

//    static {
//        ProviderManager.getInstance().addIQProvider(NAME, NAMESPACE, new Provider());
//    }

    /**
     * Creates a RTPBridge Instance with defined Session ID
     *
     * @param sid
     */
    public RTPBridge(String sid) {
        this.sid = sid;
    }

    /**
     * Creates a RTPBridge Instance with defined Session ID
     *
     * @param action
     */
    public RTPBridge(BridgeAction action) {
        this.bridgeAction = action;
    }

    /**
     * Creates a RTPBridge Instance with defined Session ID
     *
     * @param sid
     * @param bridgeAction
     */
    public RTPBridge(String sid, BridgeAction bridgeAction) {
        this.sid = sid;
        this.bridgeAction = bridgeAction;
    }

    /**
     * Creates a RTPBridge Packet without Session ID
     */
    public RTPBridge() {
    }

    /**
     * Get the attributes string
     */
    public String getAttributes() {
        StringBuilder str = new StringBuilder();

        if (getSid() != null)
            str.append(" sid='").append(getSid()).append("'");

        if (getPass() != null)
            str.append(" pass='").append(getPass()).append("'");

        if (getPortA() != -1)
            str.append(" porta='").append(getPortA()).append("'");

        if (getPortB() != -1)
            str.append(" portb='").append(getPortB()).append("'");

        if (getHostA() != null)
            str.append(" hosta='").append(getHostA()).append("'");

        if (getHostB() != null)
            str.append(" hostb='").append(getHostB()).append("'");

        return str.toString();
    }

    /**
     * Get the Session ID of the Packet (usually same as Jingle Session ID)
     *
     * @return
     */
    public String getSid() {
        return sid;
    }

    /**
     * Set the Session ID of the Packet (usually same as Jingle Session ID)
     *
     * @param sid
     */
    public void setSid(String sid) {
        this.sid = sid;
    }

    /**
     * Get the Host A IP Address
     *
     * @return
     */
    public String getHostA() {
        return hostA;
    }

    /**
     * Set the Host A IP Address
     *
     * @param hostA
     */
    public void setHostA(String hostA) {
        this.hostA = hostA;
    }

    /**
     * Get the Host B IP Address
     *
     * @return
     */
    public String getHostB() {
        return hostB;
    }

    /**
     * Set the Host B IP Address
     *
     * @param hostB
     */
    public void setHostB(String hostB) {
        this.hostB = hostB;
    }

    /**
     * Get Side A receive port
     *
     * @return
     */
    public int getPortA() {
        return portA;
    }

    /**
     * Set Side A receive port
     *
     * @param portA
     */
    public void setPortA(int portA) {
        this.portA = portA;
    }

    /**
     * Get Side B receive port
     *
     * @return
     */
    public int getPortB() {
        return portB;
    }

    /**
     * Set Side B receive port
     *
     * @param portB
     */
    public void setPortB(int portB) {
        this.portB = portB;
    }

    /**
     * Get the RTP Bridge IP
     *
     * @return
     */
    public String getIp() {
        return ip;
    }

    /**
     * Set the RTP Bridge IP
     *
     * @param ip
     */
    public void setIp(String ip) {
        this.ip = ip;
    }

    /**
     * Get the RTP Agent Pass
     *
     * @return
     */
    public String getPass() {
        return pass;
    }

    /**
     * Set the RTP Agent Pass
     *
     * @param pass
     */
    public void setPass(String pass) {
        this.pass = pass;
    }

    /**
     * Get the name of the Candidate
     *
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * Set the name of the Candidate
     *
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    public void execute(InfoQuery query, Backend backend){}

    /**
     * Get the Child Element XML of the Packet
     *
     * @return
     */
    public void appendToXML(StringBuffer str) {
        str.append("<" + ELEMENT_NAME + " xmlns='" + NAMESPACE + "' sid='").append(sid).append("'>");

        if (bridgeAction.equals(BridgeAction.create))
            str.append("<candidate/>");
        else if (bridgeAction.equals(BridgeAction.change))
            str.append("<relay ").append(getAttributes()).append(" />");
        else
            str.append("<publicip ").append(getAttributes()).append(" />");

        str.append("</" + ELEMENT_NAME + ">");

    }

    /**
     * IQProvider for RTP Bridge packets.
     * Parse receive RTPBridge packet to a RTPBridge instance
     *
     * @author Thiago Rocha
     */
    public static class Provider extends ExtensionHandler {
    	RTPBridge iq;
    	
        public Provider() {
            super();
        }
        
        public void startHandling(Attributes attr){
        	iq = new RTPBridge();
            iq.setSid(attr.getValue("sid"));
        }
        
        public void startElement(String name, Attributes attr) {
           if (name.equals("candidate")) {
               iq.setIp(attr.getValue("ip"));
               iq.setPass(attr.getValue("pass"));
               iq.setName(attr.getValue("name"));
               iq.setPortA(Integer.parseInt(attr.getValue("porta")));
               iq.setPortB(Integer.parseInt(attr.getValue("portb")));
           }
           else if (name.equals("publicip")) {
               	iq.setIp(attr.getValue("ip"));
            }
        }
        
        public Extension build(){
        	return iq;
        }
        
    }

    /**
     * Get a new RTPBridge Candidate from the server.
     * If a error occurs or the server don't support RTPBridge Service, null is returned.
     *
     * @param xmppConnection
     * @param sessionID
     * @return
     */
    public static InfoQuery getRTPBridge(Backend xmppConnection, String sessionID) {

        if (!xmppConnection.isLoggedIn()) {
            return null;
        }

        RTPBridge rtpPacket = new RTPBridge(sessionID);
        JID to = new JID(RTPBridge.NAME + "." + xmppConnection.getMyJID().getDomain());

//        PacketCollector collector = xmppConnection
//                .createPacketCollector(new PacketIDFilter(rtpPacket.getPacketID()));
//FIXME add to
        return new InfoQuery(to,"set", rtpPacket);
       

//        RTPBridge response = (RTPBridge) collector
//                .nextResult(SmackConfiguration.getPacketReplyTimeout());
//
//        // Cancel the collector.
//        collector.cancel();

       //return response;
    }

    /**
     * Check if the server support RTPBridge Service.
     *
     * @param xmppConnection
     * @return
     */
    public static void serviceAvailable(Backend xmppConnection,final Observer observer) {

        if (!xmppConnection.isLoggedIn()) {
            return;
        }

        System.out.println("Service listing");

//        ServiceDiscoveryManager disco = ServiceDiscoveryManager
//                .getInstanceFor(xmppConnection);
      
        xmppConnection.getItems(new JID(xmppConnection.getMyJID().getDomain()), new DiscoveryListener(){
			public void discoveryInfoResult(JID jid, DiscoveryInfo info){}
		
			public void discoveryItemResult(JID jid, DiscoveryItem item){
				boolean available = false;
				if (item.hasItems()){
					for(Iterator i = item.getItems();i.hasNext();){
						DiscoveryItem di = (DiscoveryItem)i.next();
						if (di.getJID().getDomain().startsWith("rtpbridge.")) {
		                    available = true;
		                    break;
		                }
					}
				}
				if(!available) System.err.println("No RTP Bridge service available");
				else if(observer!=null)observer.update(null,null);
			}
		});
    }

    /**
     * Check if the server support RTPBridge Service.
     *
     * @param xmppConnection
     * @return
     */
    public static void relaySession(Backend xmppConnection, String sessionID, String pass, TransportCandidate proxyCandidate, TransportCandidate localCandidate) {

        if (!xmppConnection.isLoggedIn()) {
            return;
        }

        RTPBridge rtpPacket = new RTPBridge(sessionID, RTPBridge.BridgeAction.change);
        JID to = new JID(RTPBridge.NAME + "." + xmppConnection.getMyJID().getDomain());
        //rtpPacket.setType(Type.SET);

        rtpPacket.setPass(pass);
        rtpPacket.setPortA(localCandidate.getPort());
        rtpPacket.setPortB(proxyCandidate.getPort());
        rtpPacket.setHostA(localCandidate.getIp());
        rtpPacket.setHostB(proxyCandidate.getIp());

        // System.out.println("Relayed to: " + candidate.getIp() + ":" + candidate.getPort());

//        PacketCollector collector = xmppConnection
//                .createPacketCollector(new PacketIDFilter(rtpPacket.getPacketID()));

        xmppConnection.send(new InfoQuery(to,"set", rtpPacket));

//        RTPBridge response = (RTPBridge) collector
//                .nextResult(SmackConfiguration.getPacketReplyTimeout());
//
//        // Cancel the collector.
//        collector.cancel();

        //return response;
    }

    /**
     * Get Public Address from the Server.
     *
     * @param xmppConnection
     * @return public IP String or null if not found
     */
    public static InfoQuery getPublicIP(Backend xmppConnection) {

        if (!xmppConnection.isLoggedIn()) {
            return null;
        }

        RTPBridge rtpPacket = new RTPBridge(RTPBridge.BridgeAction.publicip);
        JID to = new JID(RTPBridge.NAME + "." + xmppConnection.getMyJID().getDomain());
        //rtpPacket.setType(Type.SET);

        // System.out.println("Relayed to: " + candidate.getIp() + ":" + candidate.getPort());

//        PacketCollector collector = xmppConnection
//                .createPacketCollector(new PacketIDFilter(rtpPacket.getPacketID()));

         
        return new InfoQuery(to,"set", rtpPacket);
        

//        RTPBridge response = (RTPBridge) collector
//                .nextResult(SmackConfiguration.getPacketReplyTimeout());
//
//        // Cancel the collector.
//        collector.cancel();

      //  if(response == null) return null;

        

      //  return response.getIp();
    }

}
